# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# GNOME Activity Journal
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
# Copyright © 2010 Siegfried Gevatter <siegfried@gevatter.com>
# Copyright © 2010 Markus Korn <thekorn@gmx.de>
# Copyright © 2010 Randal Barlow <email.tehk@gmail.com>
# Copyright © 2010 Stefano Candori <stefano.candori@gmail.com>
# Copyright © 2020 The GNOME Activity Journal developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import cairo
import os
import gettext
import datetime
import math
import time
import threading
from dbus.exceptions import DBusException

import gi
from gi.repository import GObject, Gtk, Gio, Gdk, GdkPixbuf

try:
    gi.require_version('Gst', '1.0')
    from gi.repository import Gst
    Gst.init(None)
except Exception:
    Gst = None

from zeitgeist.datamodel import Event, Subject, StorageState

from .common import *
from . import content_objects
from .config import BASE_PATH, VERSION, settings, PluginManager, get_icon_path, get_data_path, bookmarker
from .sources import SUPPORTED_SOURCES
from . import external
from .store import STORE, get_related_events_for_uri, CLIENT

class DayLabel(Gtk.DrawingArea):

    _events = (
        Gdk.EventMask.ENTER_NOTIFY_MASK | Gdk.EventMask.LEAVE_NOTIFY_MASK |
        Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.BUTTON_MOTION_MASK |
        Gdk.EventMask.POINTER_MOTION_HINT_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK |
        Gdk.EventMask.BUTTON_PRESS_MASK
    )

    def __init__(self,date=None):
        super(DayLabel, self).__init__()

        self.matches_search = False
        self.style = self.get_style_context()
        self.style.add_class("day-label")
        self.font_name = self.get_pango_context().get_font_description().get_family()

        self.set_events(self._events)
        self.connect("draw", self.draw)
        if date:
            self.date = date
        else:
            self.date = datetime.date.today()
        self.set_size_request(100, 60)

    @property
    def date_string(self):
        return self.date.strftime("%A %d %B %Y")

    @property
    def weekday_string(self):
        if self.date == datetime.date.today():
            return _("Today")
        timedelta = datetime.date.today() -self.date
        if timedelta.days == 1:
            return _("Yesterday")
        return self.date.strftime("%A")

    @property
    def leading(self):
        if self.date == datetime.date.today():
            return True

    def set_date(self, date):
        self.date = date
        self.queue_draw()

    def set_matching(self, matching):
        self.matches_search = matching
        self.queue_draw()

    def day_text(self, widget, context):
        layout = widget.create_pango_layout(self.weekday_string)
        layout.set_font_description(Pango.FontDescription.from_string(self.font_name + " Bold 15"))
        layout_width, layout_height = layout.get_pixel_size()

        Gtk.render_layout(self.style, context, (self.width - layout_width)/2, self.height/2 - layout_height, layout)

        self.date_text(widget, context, self.height/2)

    def date_text(self, widget, context, last_font_height):
        layout = widget.create_pango_layout(self.date_string)
        layout.set_font_description(Pango.FontDescription.from_string(self.font_name + " 10"))
        layout_width, layout_height = layout.get_pixel_size()

        Gtk.render_layout(self.style, context, (self.width - layout_width)/2, last_font_height, layout)

    def draw(self, widget, context):
        if self.leading:
            self.style.add_class('white-fg-color')
        else:
            self.style.remove_class('white-fg-color')

        self.width = widget.get_allocated_width()
        self.height = widget.get_allocated_height()

        if self.leading:
            (found, bg) = self.style.lookup_color("theme_selected_bg_color")

            if found:
                red = bg.red
                green = bg.green
                blue = bg.blue
            else:
                # GNOME blue
                c = Gdk.RGBA()
                Gdk.RGBA.parse(c, "#3584e4")

                red = c.red
                green = c.green
                blue = c.blue
        else:
            (found, bg) = self.style.lookup_color("theme_base_color")

            if found:
                red = bg.red
                green = bg.green
                blue = bg.blue
            else:
                red = 1.0
                green = 1.0
                blue = 1.0

        x = 0; y = 0
        r = 5

        transparent = 1
        context.new_sub_path()
        context.arc(r+x, r+y, r, math.pi, 3 * math.pi /2)
        context.arc(self.width-r, r+y, r, 3 * math.pi / 2, 0)
        context.close_path()
        context.set_source_rgba(red, green, blue, transparent)
        context.rectangle(0, r, self.width, self.height)
        context.fill()

        color = get_gtk_rgba(self.style, "bg", 0)
        separator_red = color.red
        separator_green = color.green
        separator_blue = color.blue

        context.set_source_rgba(separator_red, separator_green, separator_blue, transparent)
        context.rectangle(0, r + self.height - 8, self.width, 8)
        context.fill()

        if self.matches_search:
            color = get_histogram_search_color(self.style)
            highlight_red = color.red
            highlight_green = color.green
            highlight_blue = color.blue
            context.set_source_rgba(highlight_red, highlight_green, highlight_blue, transparent)
            context.rectangle(0, r + self.height - 11, self.width, 3)
            context.fill()

        self.day_text(widget, context)

        return False

class DayButton(Gtk.DrawingArea):
    leading = False
    pressed = False
    today_pressed = False
    sensitive = True
    today_hover = False
    hover = False
    header_size = 60
    bg_color = (0, 0, 0, 0)
    header_color = (1, 1, 1, 1)
    leading_header_color = (1, 1, 1, 1)
    internal_color = (0, 1, 0, 1)
    arrow_color = (1, 1, 1, 1)
    arrow_color_selected = (1, 1, 1, 1)

    __gsignals__ = {
        "clicked":  (GObject.SignalFlags.RUN_LAST, None,()),
        "jump-to-today": (GObject.SignalFlags.RUN_LAST, None,()),
        }
    _events = (
        Gdk.EventMask.ENTER_NOTIFY_MASK | Gdk.EventMask.LEAVE_NOTIFY_MASK |
        Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK | Gdk.EventMask.BUTTON_PRESS_MASK |
        Gdk.EventType.MOTION_NOTIFY |   Gdk.EventMask.POINTER_MOTION_MASK
    )

    @classmethod
    def new(cls, side = 0, sensitive=True):
        button = DayButton(side, sensitive)
        def query_tooltip(widget, x, y, keyboard_mode, tooltip, ebutton):
            if not ebutton.sensitive:
                return False
            elif y < ebutton.header_size and ebutton.side == 1:
                text = _("Go to Today")
            elif y >= ebutton.header_size:
                text = _("Go to the previous day") if ebutton.side == 0 else _("Go to the next day")
            else:
                return False
            tooltip.set_text(text)
            return True
        evbox = Gtk.EventBox()
        evbox.connect("query-tooltip", query_tooltip, button)
        evbox.set_property("has-tooltip", True)
        evbox.add(button)

        return button, evbox

    def __init__(self, side = 0, sensitive=True):
        super(DayButton, self).__init__()

        self.style = self.get_style_context()
        self.style.add_class("day-button")

        self.set_events(self._events)
        self.set_can_focus(True)
        self.side = side
        self.connect("button_press_event", self.on_press)
        self.connect("button_release_event", self.clicked_sender)
        self.connect("key_press_event", self.keyboard_clicked_sender)
        self.connect("motion_notify_event", self.on_hover)
        self.connect("leave_notify_event", self._enter_leave_notify, False)
        self.connect("draw", self.draw)
        self.connect("style-updated", self.change_style)
        self.set_size_request(20, -1)
        self.set_sensitive(sensitive)

    def set_leading(self, leading):
        self.leading = leading

    def set_sensitive(self, case):
        self.sensitive = case
        self.queue_draw()

    def _enter_leave_notify(self, widget, event, bol):
        self.hover = bol
        self.today_hover = bol
        self.queue_draw()

    def on_hover(self, widget, event):
        if event.y > self.header_size:
            if not self.hover:
                self.hover = True
            else:
                self.today_hover = False
        else:
            if self.hover:
                self.hover = False
            else:
                self.today_hover = True
        self.queue_draw()
        return False

    def on_press(self, widget, event):
        if event.y > self.header_size:
            self.pressed = True
        else:
            self.today_pressed = True
        self.queue_draw()

    def keyboard_clicked_sender(self, widget, event):
        if event.keyval in (Gdk.KEY_Return, Gdk.KEY_space):
            if self.sensitive:
                self.emit("clicked")
            self.pressed = False
            self.queue_draw()
            return True
        return False

    def clicked_sender(self, widget, event):
        if event.y > self.header_size:
            if self.sensitive:
                self.emit("clicked")
        elif event.y < self.header_size:
            self.emit("jump-to-today")
        self.pressed = False
        self.today_pressed = False;
        self.queue_draw()
        return True

    def change_style(self, widget):
        self.bg_color = get_gtk_rgba(self.style, "bg", 0)
        self.header_color = get_gtk_rgba(self.style, "bg", 0, 1.25)
        self.leading_header_color = get_gtk_rgba(self.style, "bg", 3)
        self.internal_color = get_gtk_rgba(self.style, "bg", 0, 1.02)
        self.arrow_color = get_gtk_rgba(self.style, "text", 0, 0.6)
        self.arrow_color_selected = get_gtk_rgba(self.style, "bg", 3)
        self.arrow_color_insensitive = get_gtk_rgba(self.style, "text", 4)

    def draw(self, widget, context):
        width = self.get_allocated_width()
        height = self.get_allocated_height()
        context.set_source_rgba(*self.bg_color)
        context.set_operator(cairo.Operator.SOURCE)
        context.paint()
        context.rectangle(0, 0, width, height)
        context.clip()

        x = 0; y = 0
        r = 5

        w = widget.get_allocated_width()
        h = widget.get_allocated_height()

        angle = GLib.PI_2 if self.side else 3 * GLib.PI_2

        size = 20
        if self.sensitive:
            context.set_source_rgba(*(self.leading_header_color if self.leading else self.header_color))
            context.new_sub_path()
            context.move_to(x+r,y)
            context.line_to(x+w-r,y)
            context.curve_to(x+w,y,x+w,y,x+w,y+r)
            context.line_to(x+w,y+h-r)
            context.curve_to(x+w,y+h,x+w,y+h,x+w-r,y+h)
            context.line_to(x+r,y+h)
            context.curve_to(x,y+h,x,y+h,x,y+h-r)
            context.line_to(x,y+r)
            context.curve_to(x,y,x,y,x+r,y)
            # What's this for, exactly? Appears to have been already set.
            #context.set_source_rgba(*(self.leading_header_color if self.leading else self.header_color))
            context.close_path()
            context.rectangle(0, r, w,  self.header_size)
            context.fill()
            context.set_source_rgba(*self.internal_color)
            context.rectangle(0, self.header_size, w,  h)
            context.fill()
            if self.hover:
                Gtk.render_frame(self.style, context,
                                 0, self.header_size,
                                 w, h-self.header_size)
            if self.side > 0 and self.today_hover:
                Gtk.render_frame(self.style, context,
                                 0, 0,
                                 w, self.header_size)
        size = 10
        if not self.sensitive:
            state = Gtk.StateFlags.INSENSITIVE
        elif self.is_focus() or self.pressed:
            Gtk.render_focus(self.style, context,
                             0, self.header_size,
                             w, h-self.header_size)
            state = Gtk.StateFlags.SELECTED
        else:
            state = Gtk.StateFlags.NORMAL
        Gtk.render_arrow(self.style, context, angle,
                         w/2-size/2, h/2 + size/2, size)
        size = 7

        # Paint today button arrows.
        if self.sensitive and self.side > 0:
            if self.today_hover:
                if self.today_pressed:
                    Gtk.render_arrow(self.style, context, angle,
                                     w/2, self.header_size/2 - size/2, size)
                    Gtk.render_arrow(self.style, context, angle,
                                     w/2-size/2, self.header_size/2 - size/2, size)
                else:
                    Gtk.render_arrow(self.style, context, angle,
                                     w/2, self.header_size/2 - size/2, size)
                    Gtk.render_arrow(self.style, context, angle,
                                     w/2-size/2, self.header_size/2 - size/2, size)
            else:
                Gtk.render_arrow(self.style, context, angle,
                                 w/2, self.header_size/2 - size/2, size)
                Gtk.render_arrow(self.style, context, angle,
                                 w/2-size/2, self.header_size/2 - size/2, size)

class SearchBox(Gtk.ToolItem):

    __gsignals__ = {
        "clear" : (GObject.SignalFlags.RUN_FIRST,
                   None,
                   ()),
        "search" : (GObject.SignalFlags.RUN_FIRST,
                    None,
                    (GObject.TYPE_PYOBJECT,)),
        "matching-day" : (GObject.SignalFlags.RUN_FIRST,
                          None,
                          (GObject.TYPE_PYOBJECT,)),
        "matching-days" : (GObject.SignalFlags.RUN_FIRST,
                           None,
                           (GObject.TYPE_PYOBJECT,)),
    }

    search_cleared = False
    search_in_pending = False
    search_in_progress = False

    @property
    def use_fts(self):
        #if STORE.fts_search_enabled:
        #    return self.fts_checkbutton.get_active()
        return False

    def __init__(self):
        Gtk.ToolItem.__init__(self)

        self.style = self.get_style_context()
        self.style.add_class("search-box")

        self.text = ""
        self.previous_text = ""
        self.callback = None
        self.set_border_width(3)
        self.hbox = Gtk.HBox(False, 6)
        self.search_hbox = Gtk.HBox(False, 0)
        self.add(self.hbox)
        self.results = []
        self.search = Gtk.SearchEntry()
        self.search_style = self.search.get_style_context()
        self.search.set_placeholder_text(_("Type to search"))
        self.search.set_width_chars(35);
        self.category = {}
        self.matching_dates = None
        self.matching_date_index = 0
        self.total_matching_dates = 0
        self.search_progress_timeout = 0
        self.warning_timeout = 0

        #if STORE.fts_search_enabled:
        #    self.fts_checkbutton = Gtk.CheckButton(_("Use Zeitgeist FTS"))

        for source in list(SUPPORTED_SOURCES.keys()):
            s = SUPPORTED_SOURCES[source]._desc_pl
            self.category[s] = source
        self.combobox = Gtk.ComboBoxText()

        self.padding = Gtk.Label()
        self.padding.set_width_chars(10)

        self.label = Gtk.Label()
        self.label.set_width_chars(30)
        self.label.set_alignment(0.0, 0.5)

        # Previous matching day button
        self.previous_button = previous_button = Gtk.Button.new_from_icon_name("go-previous-symbolic", Gtk.IconSize.BUTTON)
        previous_button.connect("clicked", self.on_previous_match_clicked)
        previous_button.set_tooltip_text(_("Go to the previous matching day"))
        previous_button.set_sensitive(False)

        # Next matching day button
        self.next_button = next_button = Gtk.Button.new_from_icon_name("go-next-symbolic", Gtk.IconSize.BUTTON)
        next_button.connect("clicked", self.on_next_match_clicked)
        next_button.set_sensitive(False)
        next_button.set_tooltip_text(_("Go to the next matching day"))

        self.search_hbox.get_style_context().add_class("linked")

        self.combobox.set_focus_on_click(False)
        self.search_hbox.pack_start(self.search, False, True, 0)
        self.search_hbox.pack_start(self.previous_button, False, True, 0)
        self.search_hbox.pack_start(self.next_button, False, True, 0)

        self.hbox.pack_start(self.padding, False, True, 0)
        self.hbox.pack_start(self.combobox, False, False, 0)
        self.hbox.pack_start(self.search_hbox, False, True, 0)
        self.hbox.pack_start(self.label, False, False, 0)
        #if STORE.fts_search_enabled:
        #    self.hbox.pack_end(self.fts_checkbutton, False, True, 0)
        self.combobox.append_text("All activities")
        self.combobox.set_active(0)
        for cat in list(self.category.keys()):
            self.combobox.append_text(cat)

        def combo_changed(widget, self):
            self.set_search(self.search)

        self.combobox.connect("changed", combo_changed, self)
        self.show_all()

        def change_style(widget):
            color = get_gtk_rgba(self.style, "bg", 0)
            fcolor = get_gtk_rgba(self.style, "fg", 0)

            color = shade_gdk_color(color, 102/100.0)
            self.override_background_color(Gtk.StateFlags.NORMAL, color)

            color = combine_gdk_color(color, fcolor)
            self.search.override_color(Gtk.StateFlags.NORMAL, color)

        # self.hbox.connect("style-updated", change_style)
        self.search.connect("search-changed", self.set_search)

    def update_navigation_label(self):
        # do not update ui if another search is in progress or search
        # is cleared.
        if self.ignore_search("labelling", self.previous_text):
            self.search_finished()
            return

        label_uris = gettext.ngettext("{} match", "{} matches", self.total_matching_uri).format(self.total_matching_uri)
        label_days = gettext.ngettext("{} day", "{} days", self.total_matching_dates).format(self.total_matching_dates)
        label_events = gettext.ngettext("{} event", "{} events", self.total_valid_events).format(self.total_valid_events)
        link = gettext.ngettext(" in ", " over ", self.total_matching_dates)

        display_label = label_uris + link + label_days
        tooltip_label = label_events

        self.label.set_text(display_label)
        self.label.set_tooltip_text(tooltip_label)

    def reset_navigation(self):
        self.previous_button.set_sensitive(False)
        self.next_button.set_sensitive(False)

        if self.total_matching_dates:
            if self.total_matching_dates > 1:
                self.previous_button.set_sensitive(True)
            self.update_navigation_label()

    def update_navigation(self):
        self.update_navigation_label()

        if self.matching_date_index == self.total_matching_dates - 1:
            self.previous_button.set_sensitive(False)
        else:
            self.previous_button.set_sensitive(True)

        if self.matching_date_index == 0:
            self.next_button.set_sensitive(False)
        else:
            self.next_button.set_sensitive(True)

    def get_previous_match(self):
        self.matching_date_index +=1
        date = self.matching_dates[self.matching_date_index]
        return date

    def get_next_match(self):
        self.matching_date_index -=1
        date = self.matching_dates[self.matching_date_index]
        return date

    def on_previous_match_clicked(self, button):
        date = self.get_previous_match()
        self.update_navigation()
        self.emit("matching-day", date)

    def on_next_match_clicked(self, button):
        date = self.get_next_match()
        self.update_navigation()
        self.emit("matching-day", date)

    def single_char_search_warning(self):
        if self.dont_search:
            self.search_style.add_class('warning')
        self.warning_timeout = 0

    def set_search(self, entry):
        text = entry.get_text()
        self.text = text

        # clear error style if no matches found
        self.search_style.remove_class('error')
        self.search_style.remove_class('warning')

        interpretation = None
        self.matching_dates = None
        self.total_matching_uri = 0
        self.total_matching_dates = 0
        self.total_valid_events = 0
        self.matching_date_index = 0

        no_search = False
        self.dont_search = False

        if config.SEARCH_IGNORE_SINGLE_CHAR and len(text) == 1:
            self.dont_search = True
            if self.warning_timeout:
                GObject.source_remove(self.warning_timeout)
            self.warning_timeout = GLib.timeout_add(1000, self.single_char_search_warning)

        if not text:
            no_search = True

        if self.dont_search or no_search:
            self.results = []
            self.emit("clear")
            self.label.set_text("")
            self.label.set_tooltip_text("")
            SearchBox.search_cleared = True

            if config.DEBUG:
                print("all search cleared")

            # make sure to clear the flag in content objects
            content_objects.ContentObject.clear_search_matches()
            self.reset_navigation()
            return

        SearchBox.search_cleared = False

        def callback(text, results):
            self.results = results

            # mark first before emitting 'search' signal
            marked = self.__mark_search_results(text, results)
            if marked:
                self.emit("search", results)

        cat = self.combobox.get_active()
        if cat != 0:
            cat = self.category[self.combobox.get_active_text()]
            interpretation = self.category[self.combobox.get_active_text()]
            if interpretation:
                return self.do_search(text, callback, interpretation)

        return self.do_search(text, callback)

    def do_search(self, text, callback=None, interpretation=None):
        if not callback: return
        self.do_search_objs(text, callback, interpretation)

    def search_progress(self):
        self.search.progress_pulse()
        return GLib.SOURCE_CONTINUE

    def search_finished(self):
        self.search.set_progress_fraction(0.0)
        if self.search_progress_timeout:
            GObject.source_remove(self.search_progress_timeout)
            self.search_progress_timeout = 0

    def do_search_objs(self, text, callback, interpretation=None):
        def _search(text, callback):
            if SearchBox.search_in_progress:
                if config.DEBUG:
                    print ("search '%s' in progress. re-scheduling" % (self.previous_text))
                GLib.timeout_add(200, _search, text, callback)
                SearchBox.search_in_pending = True
                return

            self.previous_text = text
            SearchBox.search_in_pending = False
            SearchBox.search_in_progress = True
            _do_search(text, callback)
            SearchBox.search_in_progress = False

        def _do_search(text, callback):
            if config.DEBUG:
                print ("searching for '%s'" % (text))

            # display search progress
            self.search.set_progress_fraction(0.1)
            self.search_progress_timeout = GLib.timeout_add(100, self.search_progress)

            if STORE.fts_search_enabled and self.use_fts:
                matching = STORE.search_using_zeitgeist_fts(text, [Event.new_for_values(subject_interpretation=interpretation)] if interpretation else [])
            else:
                def matching_test_function(obj):
                    subject = obj.event.subjects[0]

                    # match filenames
                    if not config.SEARCH_IS_CASE_SENSITIVE:
                        match_name = text.lower() in subject.text.lower()
                    else:
                        match_name = text in subject.text

                    match_uri = False
                    # match path in uri
                    if config.SEARCH_IN_SUBJECT_URI and not match_name:
                        if not config.SEARCH_IS_CASE_SENSITIVE:
                            match_uri = text.lower() in subject.uri.lower()
                        else:
                            match_uri = text in subject.uri

                    if match_name or match_uri:
                        if interpretation:
                            try:
                                if subject.interpretation != interpretation:
                                    return False
                            except Exception:
                                return False
                        return True
                    return False
                matching = STORE.search_store_using_matching_function(matching_test_function)

            # do not update ui if another search is in progress or search
            # is cleared.
            if self.ignore_search("search-completion", text):
                self.search_finished()
                return

            if config.DEBUG:
                print("updating search results for '%s'" % (text))

            callback(text, matching)

        GLib.idle_add(_search, text, callback)

    def do_search_using_zeitgeist(self, text, callback=None, interpretation=""):
        if not text: return
        self.callback = callback
        templates = [
            Event.new_for_values(subject_text="*"+text+"*", subject_interpretation=interpretation),
            Event.new_for_values(subject_uri="*"+text+"*", subject_interpretation=interpretation)
        ]
        CLIENT.find_event_ids_for_templates(templates, self._search_callback,
            storage_state=StorageState.Available, num_events=20, result_type=0)

    def _search_callback(self, ids):
        objs = []
        for id_ in ids:
            try:
                obj.append(STORE[id_])
            except KeyError:
                continue
        if self.callback:
            self.callback(objs)

    def toggle_visibility(self):
        if self.get_property("visible"):
            self.hide()
            return False
        self.show()
        return True

    def ignore_search(self, marker, text):
        if SearchBox.search_in_pending or SearchBox.search_cleared:
            if config.DEBUG:
                print("[%s] - ignoring search results for '%s' as new search "
                      "is in progress" % (marker, text))
            return True

        if config.DEBUG:
            print("[%s] - search '%s' proceeding" % (marker, text))

        return False

    def __mark_search_results(self, text, results):
        content_objects.ContentObject.clear_search_matches()

        unique_days = []
        unique_uris = []
        timestamps_millis = []

        unique_day_count = 0
        unique_uri_count = 0
        valid_result_count = 0
        all_result_count = len(results)

        for obj in results:
            if obj.content_object:
                # do not update ui if another search is in progress or
                # search is cleared.
                if self.ignore_search("marking", text):
                    self.search_finished()
                    content_objects.ContentObject.clear_search_matches()
                    return False

                valid_result_count += 1

                # non-unique timestamp is needed for day spread count
                timestamp_millis = int(obj.event.timestamp)
                timestamps_millis.append(timestamp_millis)
                timestamp_secs = timestamp_millis/1000.0

                # unique uris
                if obj.content_object.uri not in unique_uris:
                    matching_day = datetime.date.fromtimestamp(timestamp_millis/1000)
                    unique_uris.append(obj.content_object.uri)
                    unique_uri_count += 1

                # mark as match
                setattr(obj.content_object, "matches_search", True)

        # get all matching dates
        timestamps_millis = list(set(timestamps_millis))
        timestamps_millis.sort(reverse=True)

        for timestamp in timestamps_millis:
            matching_day = datetime.date.fromtimestamp(timestamp/1000)
            if matching_day not in unique_days:
                unique_day_count += 1
                unique_days.append(matching_day)

        self.matching_dates = unique_days
        self.total_matching_uri = unique_uri_count
        self.total_matching_dates = unique_day_count
        self.total_valid_events = valid_result_count
        self.matching_date_index = 0

        self.emit("matching-days", self.matching_dates)
        if unique_day_count:
            self.emit("matching-day", self.matching_dates[0])
        else:
            self.search_style.add_class('error')
            self.label.set_tooltip_text("")
            self.label.set_text("")

        if config.DEBUG:
            print("matches for text '%s': %d (all), %d (valid), %d (unique uris), %d (unique days)" %
                  (text, all_result_count, valid_result_count, unique_uri_count, unique_day_count))

        self.reset_navigation()
        self.search_finished()

        return True

class PreviewTooltip(Gtk.Window):

    # per default we are using thumbs at a size of 128 * 128 px
    # in tooltips. For preview of text files we are using 256 * 256 px
    # which is dynamically defined in StaticPreviewTooltip.preview()
    TOOLTIP_SIZE = SIZE_NORMAL

    def __init__(self):
        Gtk.Window.__init__(self, type=Gtk.WindowType.POPUP)

    def preview(self, gio_file):
        return False


class StaticPreviewTooltip(PreviewTooltip):

    def __init__(self):
        super(StaticPreviewTooltip, self).__init__()
        self.__current = None
        self.__monitor = None

    def replace_content(self, content):
        children = self.get_children()
        if children:
            self.remove(children[0])
            # hack to force the tooltip to have the exact same size
            # as the child image
            self.resize(1,1)
        self.add(content)

    def preview(self, gio_file):
        if gio_file == self.__current:
            return bool(self.__current)
        if self.__monitor is not None:
            self.__monitor.cancel()
        self.__current = gio_file
        self.__monitor = gio_file.get_monitor()
        self.__monitor.connect("changed", self._do_update_preview)
        # for text previews we are always using SIZE_LARGE
        if "text-x-generic" in gio_file.icon_names or "text-x-script" in gio_file.icon_names:
            size = SIZE_LARGE
        else:
            size = self.TOOLTIP_SIZE
        if not isinstance(gio_file, GioFile): return False
        pixbuf = gio_file.get_thumbnail(size=size, border=1)
        if pixbuf is None:
            self.__current = None
            return False
        img = Gtk.Image.new_from_pixbuf(pixbuf)
        img.set_alignment(0.5, 0.5)
        img.show_all()
        self.replace_content(img)
        del pixbuf, size
        return True

    def _do_update_preview(self, monitor, file, other_file, event_type):
        if event_type == Gio.FileMonitorEvent.CHANGES_DONE_HINT:
            if self.__current is not None:
                self.__current.refresh()
            self.__current = None
            Gtk.tooltip_trigger_tooltip_query(Gdk.Display.get_default())


class VideoPreviewTooltip(PreviewTooltip):

    def __init__(self):
        PreviewTooltip.__init__(self)
        hbox = Gtk.HBox()
        self.movie_window = Gtk.DrawingArea()
        hbox.pack_start(self.movie_window, False, True, 0)
        self.add(hbox)
        self.has_preview = True
        self.player = Gst.ElementFactory.make("playbin", "player")
        self.sink = Gst.ElementFactory.make("gtksink", "gtksink")

        self.showing = False

        # no video preview if elements are missing -or- if user doesn't want it
        if self.player == None or self.sink == None or not config.SHOW_VIDEO_PREVIEW:
            self.has_preview = False
            return

        self.player.props.video_sink = self.sink
        self.gst_widget = self.sink.props.widget

        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.on_message)
        self.connect("hide", self._handle_hide)
        self.connect("show", self._handle_show)
        # FixMe: hack to hide the preview window. rework this.
        self.set_default_size(0, 0)
        self.gst_widget.set_size_request(*SIZE_LARGE)
        self.window = None

        display = Gdk.Display.get_default()
        monitor = display.get_monitor(0) # Monitor 0 for now
        geometry = monitor.get_geometry()

        self.monitor_width = geometry.width
        self.monitor_height = geometry.height

    @staticmethod
    def check_dependencies():
        player = Gst.ElementFactory.make("playbin", "player")
        sink = Gst.ElementFactory.make("gtksink", "gtksink")

        # no video preview if elements are missing
        return (player and sink)

    def has_video_preview(self):
        return self.has_preview

    def _handle_hide(self, widget):
        if not self.has_preview:
            return

        self.showing = False
        self.hide()
        self.window.hide()
        self.player.set_state(Gst.State.NULL)

    def _handle_show(self, widget):
        if not self.has_preview:
            return

        self.showing = True
        self.show_all()
        self.window.show_all()
        self.player.set_state(Gst.State.PLAYING)

    def preview(self, gio_file, x, y):
        if not self.has_preview:
            return

        frame_w = 320
        frame_h = 240

        # Use custom gst gtk window
        if self.window:
            self.window.remove(self.gst_widget)
        self.window = window = Gtk.Window()

        window.set_default_size(frame_w, frame_h)
        window.add(self.gst_widget)
        window.set_title(_("Video Preview"))

        x_offset = 50
        y_offset = 50

        if self.monitor_width - x < frame_w + 3 * x_offset:
            x_offset = -(frame_w + 3 * x_offset)

        if self.monitor_height - y < frame_h + 3 * y_offset:
            y_offset = -(frame_h + 3 * y_offset)

        # hope window manager honors the request.
        self.window.move(x + x_offset, y + y_offset)

        if gio_file.uri == self.player.get_property("uri"):
            return True
        self.player.set_property("uri", gio_file.uri)
        return True

    def on_message(self, bus, message):
        t = message.type
        if t == Gst.MessageType.EOS:
            self.player.set_state(Gst.State.NULL)
            self.hide()
        elif t == Gst.MessageType.ERROR:
            self.player.set_state(Gst.State.NULL)
            err, debug = message.parse_error()
            print("Error: %s" % err, debug)

class AudioPreviewTooltip(PreviewTooltip):

    def __init__(self):
        PreviewTooltip.__init__(self)
        #Playing label stuffs

        if False:
            screen = self.get_screen()
            map_ = screen.get_rgba_colormap()
            if map_ is None:
                map_ = screen.get_rgb_colormap()
                self.set_colormap(map_)

        self.has_preview = True
        self.showing = False

        # GStreamer stuffs
        self.player = Gst.ElementFactory.make("playbin", "player")
        fakesink = Gst.ElementFactory.make("fakesink", "fakesink")

        # no audio preview if gstreamer elements are missing -or- if user doesn't want it
        if self.player == None or fakesink == None or not config.SHOW_AUDIO_PREVIEW:
            self.has_preview = False
            return

        self.set_app_paintable(True)
        img = Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PLAY,Gtk.IconSize.LARGE_TOOLBAR)
        self.image = AnimatedImage(get_data_path("zlogo/zg%d.png"), 150, size=20)
        self.image.set_valign(Gtk.Align.CENTER)
        self.image.start()
        label = Gtk.Label()
        label.set_markup(_("<b><span color='white'>Audio Preview</span></b>"))
        label.set_valign(Gtk.Align.CENTER)
        hbox = Gtk.HBox()
        hal = Gtk.Alignment.new(0.5, 0.5, 1.0, 1.0)
        hal.set_padding(12, 12, 0, 6)
        hal.add(label)
        hbox.pack_start(self.image, False, True, 6)
        hbox.pack_end(hal, False, True, 0)
        self.resize(1,1)
        self.add(hbox)

        self.player.set_property("video-sink", fakesink)
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.connect("message", self.on_message)
        self.connect("hide", self._handle_hide)
        self.connect("show", self._handle_show)
        self.connect("draw", self.transparent_draw)

    def transparent_draw(self, widget, context):
        window = widget.props.window
        context = window.cairo_create()
        context.set_source_rgba(0, 0, 0, 0.75)
        context.paint()

        return False

    @staticmethod
    def check_dependencies():
        player = Gst.ElementFactory.make("playbin", "player")
        fakesink = Gst.ElementFactory.make("fakesink", "fakesink")

        # no audio preview if elements are missing
        return (player and fakesink)

    def has_audio_preview(self):
        return self.has_preview

    def _handle_hide(self, widget):
        if not self.has_preview:
            return

        self.showing = False
        self.image.stop()
        self.player.set_state(Gst.State.NULL)

    def _handle_show(self, widget):
        if not self.has_preview:
            return

        self.showing = True
        self.image.start()
        self.show_all()
        self.player.set_state(Gst.State.PLAYING)

    def preview(self, gio_file, x=0, y=0):
        if not self.has_preview:
            return

        if gio_file.uri == self.player.get_property("uri"):
            return True
        self.player.set_property("uri", gio_file.uri)
        return True

    def on_message(self, bus, message):
        t = message.type
        if t == Gst.MessageType.EOS:
            self.player.set_state(Gst.State.NULL)
        elif t == Gst.MessageType.ERROR:
            self.player.set_state(Gst.State.NULL)
            err, debug = message.parse_error()
            print("Error: %s" % err, debug)

    def replace_content(self, content):
        children = self.get_children()
        if children:
            self.remove(children[0])
            # hack to force the tooltip to have the exact same size
            # as the child image
            self.resize(1,1)
        self.add(content)

class AnimatedImage(Gtk.Image):
    animating = None
    mod = 7
    i = 0
    speed = 100
    def __init__(self, uri, speed = 0, size = 16):
        super(AnimatedImage, self).__init__()
        if speed: self.speed = speed
        self.frames = []
        for i in (6, 5, 4, 3, 2, 1, 0):
            self.frames.append(GdkPixbuf.Pixbuf.new_from_file_at_size(get_icon_path(uri % i), size, size))
        self.set_from_pixbuf(self.frames[0])

    def next(self):
        """
        Move to next frame
        """
        self.set_from_pixbuf(self.frames[self.i % self.mod])
        self.i += 1
        return True

    def start(self):
        """
        start the image's animation
        """
        if self.animating: GObject.source_remove(self.animating)
        self.animating = GObject.timeout_add(self.speed, self.next)

    def stop(self):
        """
        stop the image's animation
        """
        if self.animating: GObject.source_remove(self.animating)
        self.animating = None
        return False

    def animate_for_seconds(self, seconds):
        """
        :param seconds: int seconds for the amount of time when you want
        animate the throbber
        """
        self.start()
        GObject.timeout_add_seconds(seconds, self.stop)


class ThrobberPopupButton(Gtk.ToolItem):

    def __init__(self):
        super(ThrobberPopupButton, self).__init__()
        self.button = button = Gtk.MenuButton()
        self.button.set_direction(Gtk.ArrowType.NONE)
        self.button.set_valign(Gtk.Align.CENTER)
        self.button.set_margin_end(6)
        self.add(button)

        builder = Gtk.Builder.new_from_file(get_data_path("ui/app-menu.ui"))
        menu_model = builder.get_object("app-menu")
        self.popover = Gtk.Popover.new_from_model(button, menu_model)

        button.set_popover(self.popover)

class AboutDialog(Gtk.AboutDialog):
    name = "Activity Journal"
    authors = (
        "Seif Lotfy <seif@lotfy.com>",
        "Randal Barlow <email.tehk@gmail.com>",
        "Siegfried-Angel Gevatter <siegfried@gevatter.com>",
        "Peter Lund <peterfirefly@gmail.com>",
        "Hylke Bons <hylkebons@gmail.com>",
        "Markus Korn <thekorn@gmx.de>",
        "Mikkel Kamstrup <mikkel.kamstrup@gmail.com>",
        "Thorsten Prante <thorsten@prante.eu>",
        "Stefano Candori <stefano.candori@gmail.com>",
        "crvi <crvisqr@gmail.com>"
    )

    artists = (
        "Hylke Bons <hylkebons@gmail.com>",
        "Thorsten Prante <thorsten@prante.eu>"
    )

    client_version = "{}.{}.{}".format(*external.CLIENT_VERSION)
    copyright = "Copyright 2009-2020 - The Activity Journal authors"
    comment = _("Activity Journal for the GNOME desktop environment\nPowered by Zeitgeist {}".format(client_version))
    website_label = _("Activity Journal website")
    website = "https://wiki.gnome.org/Apps/ActivityJournal"
    version = VERSION

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_program_name(self.name)
        self.set_version(self.version)
        self.set_comments(self.comment)
        self.set_copyright(self.copyright)
        self.set_authors(self.authors)
        self.set_artists(self.artists)
        self.set_license_type(Gtk.License.GPL_3_0)
        self.set_translator_credits(_("translator-credits"))
        self.set_website(self.website)
        self.set_website_label(self.website_label)

        self.set_logo_icon_name("org.gnome.ActivityJournal")
        #self.set_logo(GdkPixbuf.Pixbuf.new_from_file_at_size(get_icon_path(
         #   APP_ICON_NORMAL), 128, 128))


class ContextMenu(Gtk.Menu):
    subjects = []# A list of Zeitgeist event uris
    infowindow = None
    parent_window = None

    def __init__(self):
        super(ContextMenu, self).__init__()
        self.menuitems = {
            "open" : Gtk.ImageMenuItem(_("Open")),
            "unpin" : Gtk.MenuItem(_("Remove Pin")),
            "pin" : Gtk.MenuItem(_("Add Pin")),
            "delete" : get_menu_item_with_stock_id_and_text(Gtk.STOCK_DELETE, _("Delete item from Journal")),
            "delete_uri" : Gtk.MenuItem(_("Delete all events with this URI")),
            "info" : get_menu_item_with_stock_id_and_text(Gtk.STOCK_INFO, _("More Information")),
            }
        callbacks = {
            "open" : self.do_open,
            "unpin" : self.do_unbookmark,
            "pin" : self.do_bookmark,
            "delete" : self.do_delete,
            "delete_uri" : self.do_delete_events_with_shared_uri,
            "info" : self.do_show_info,
            }
        names = ["open", "unpin", "pin", "delete", "delete_uri", "info"]
        if is_command_available("nautilus-sendto"):
            self.menuitems["sendto"] = get_menu_item_with_stock_id_and_text(Gtk.STOCK_CONNECT, _("Send To..."))
            callbacks["sendto"] = self.do_send_to
            names.append("sendto")
        for name in names:
            item = self.menuitems[name]
            self.append(item)
            item.connect("activate", callbacks[name])
        self.show_all()
        self.confirmed = False

    def do_popup(self, time, subjects):
        """
        Call this method to popup the context menu

        :param time: the event time from the button press event
        :param subjects: a list of uris
        """
        self.subjects = subjects
        if len(subjects) == 1:
            uri = subjects[0].uri
            if bookmarker.is_bookmarked(uri):
                self.menuitems["pin"].hide()
                self.menuitems["unpin"].show()
            else:
                self.menuitems["pin"].show()
                self.menuitems["unpin"].hide()

        self.popup(None, None, None, None, 3, time)

    def do_open(self, menuitem):
        for obj in self.subjects:
            obj.launch()

    def do_show_info(self, menuitem):
        if self.subjects:
            if self.infowindow:
                self.infowindow.destroy()
            self.infowindow = InformationContainer(parent=self.parent_window)
            self.infowindow.set_content_object(self.subjects[0])
            self.infowindow.show_all()

    def do_bookmark(self, menuitem):
        for obj in self.subjects:
            uri = obj.uri
            uri = str(uri)
            isbookmarked = bookmarker.is_bookmarked(uri)
            if not isbookmarked:
                bookmarker.bookmark(uri)

    def do_unbookmark(self, menuitem):
        for obj in self.subjects:
            uri = obj.uri
            uri = str(uri)
            isbookmarked = bookmarker.is_bookmarked(uri)
            if isbookmarked:
                bookmarker.unbookmark(uri)

    def do_delete(self, menuitem):
        for obj in self.subjects:
            self.do_delete_object(obj, False)

    # Delete all matching events for object within its day part time range
    def do_delete_object(self, obj, confirmed=True):
        if obj is None: return

        self.confirmed = confirmed
        self.delete_all = False
        self.deleted_uri = obj.uri

        timerange=DayParts.get_day_part_range_for_item(obj)
        self.from_time = int(timerange[0])/1000
        self.to_time = int(timerange[1])/1000

        CLIENT.find_event_ids_for_template(
            Event.new_for_values(subject_uri=obj.uri),
            self._delete_objects,
            timerange=timerange)

    # Delete all matching events for object
    def do_delete_events_with_shared_uri(self, menuitem):
        self.delete_all = True

        for uri in [obj.uri for obj in self.subjects]:
            self.deleted_uri = uri
            CLIENT.find_event_ids_for_template(
                Event.new_for_values(subject_uri=uri),
                self._delete_objects)

    # confirm deletion and proceed
    def _delete_objects(self, ids):
        ids = list(map(int, ids))
        count = len(ids)

        # confirm deletion
        if not self.confirmed and config.CONFIRM_ITEM_DELETION:
            delete = confirm_object_deletion(self.get_toplevel(),
                                             self.deleted_uri,
                                             self.delete_all,
                                             len(ids))
            if not delete:
                return

        if config.DEBUG:
            if self.delete_all:
                print("Deleting all '%d' events for '%s'" %
                      (count, get_readable_uri(self.deleted_uri)))
            else:
                print("Deleting '%d' matching events for '%s' in time range [%s to %s]" %
                      (count, get_readable_uri(self.deleted_uri),
                       time.ctime(self.from_time),
                       time.ctime(self.to_time)))

        CLIENT.delete_events(ids)

    def do_send_to(self, menuitem):
        launch_command("nautilus-sendto", [obj.uri for obj in self.subjects])

    def set_parent_window(self, parent):
        self.parent_window = parent

class ContextMenuMolteplicity(Gtk.Menu):
    subjects = []# A list of Zeitgeist event uris
    infowindow = None
    parent_window = None

    def __init__(self):
        super(ContextMenuMolteplicity, self).__init__()
        self.menuitems = {
            "delete" : get_menu_item_with_stock_id_and_text(Gtk.STOCK_DELETE, _("Delete items from Journal")),
            "info" : get_menu_item_with_stock_id_and_text(Gtk.STOCK_INFO, _("Show all grouped items"))
            }
        callbacks = {
            "delete" : self.do_delete,
            "info" : self.do_show_info
            }
        names = ["delete", "info"]
        for name in names:
            item = self.menuitems[name]
            self.append(item)
            item.connect("activate", callbacks[name])
        self.show_all()

    def do_popup(self, time, subjects):
        """
        Call this method to popup the context menu

        :param time: the event time from the button press event
        :param subjects: a list of uris
        """
        self.subjects = subjects
        if len(subjects) == 1:
            uri = subjects[0].uri

        self.popup(None, None, None, None, 3, time)

    def do_show_info(self, menuitem):
        if self.subjects:
            self.do_show_molteplicity_list(self.subjects)

    def do_show_molteplicity_list(self, item_list):
        if item_list:
            if self.infowindow:
                self.infowindow.destroy()
            self.infowindow = MolteplicityInformationContainer(parent=self.parent_window)
            self.infowindow.set_item_list(item_list)
            self.infowindow.show_all()

    def do_delete(self, menuitem):
        for obj_ in self.subjects:
            obj = obj_.content_object
            CLIENT.find_event_ids_for_template(
                Event.new_for_values(subject_uri=obj.uri),
                lambda ids: CLIENT.delete_events(list(map(int, ids))),
                timerange=DayParts.get_day_part_range_for_item(obj))

    def do_delete_list(self, list_):
        for obj_ in list_:
            obj = obj_.content_object
            CLIENT.find_event_ids_for_template(
                Event.new_for_values(subject_uri=obj.uri),
                lambda ids: CLIENT.delete_events(list(map(int, ids))),
                timerange=DayParts.get_day_part_range_for_item(obj))


    def set_parent_window(self, parent):
        self.parent_window = parent


class ToolButton(Gtk.RadioToolButton):
    def __init__(self, *args, **kwargs):
        super(ToolButton, self).__init__(*args, **kwargs)

    def set_label(self, text):
        super(ToolButton, self).set_label(text)
        self.set_tooltip_text(text)

    @staticmethod
    def new_from_widget(widget):
        return Gtk.RadioToolButton.new_from_widget(widget)

    def set_tooltip_text(self, text):
        Gtk.Widget.set_tooltip_text(self, text)

class InformationToolButton(Gtk.ToolButton):
    def __init__(self, icon=None, *args, **kwargs):
        super(InformationToolButton, self).__init__(*args, **kwargs)
        if icon:
            image = Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.SMALL_TOOLBAR)
            self.set_icon_widget(image)

    def set_label(self, text):
        super(InformationToolButton, self).set_label(text)
        self.set_tooltip_text(text)

    def set_tooltip_text(self, text):
        Gtk.Widget.set_tooltip_text(self, text)


class Toolbar(Gtk.HBox):

    first_tool_button = None

    __gsignals__ = {
        "previous" : (GObject.SignalFlags.RUN_FIRST,
                   None,
                   ()),
        "jump-to-today" : (GObject.SignalFlags.RUN_FIRST,
                   None,
                   ()),
        "jump-to-selected-day" : (GObject.SignalFlags.RUN_FIRST,
                   None,
                   (GObject.TYPE_PYOBJECT,)),
        "next" : (GObject.SignalFlags.RUN_FIRST,
                   None,
                   ()),
    }

    @staticmethod
    def get_toolbutton(path, label_string, radio=True):
        if radio:
            # FixMe: not the right way, find the reason for crash, and
            # rework this.
            if not Toolbar.first_tool_button:
                button = ToolButton()
                Toolbar.first_tool_button = button
            else:
                button = ToolButton.new_from_widget(Toolbar.first_tool_button)

        else:
            button = InformationToolButton()
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
        image = Gtk.Image()
        image.set_from_pixbuf(pixbuf)
        button.set_icon_widget(image)
        button.set_tooltip_text(label_string)
        return button

    def __init__(self):
        super(Toolbar, self).__init__()

        # Previous-day button
        self.previousd_button = previous_button = Gtk.Button.new_from_icon_name("go-previous-symbolic", Gtk.IconSize.BUTTON)
        previous_button.connect("clicked", lambda x: self.emit("previous"))
        previous_button.set_tooltip_text(_("Go to the previous day"))

        # Next-day button
        self.nextd_button = next_button = Gtk.Button.new_from_icon_name("go-next-symbolic", Gtk.IconSize.BUTTON)
        next_button.connect("clicked", lambda x: self.emit("next"))
        next_button.set_sensitive(False)
        next_button.set_tooltip_text(_("Go to the next day"))

        navigation_box = Gtk.HBox()
        navigation_box.get_style_context().add_class("linked")

        for item in (previous_button, next_button):
            navigation_box.pack_start(item, False, False, 0)

        # Jump-to-today button
        self.home_button = home_button = Gtk.Button.new_from_icon_name("go-home-symbolic", Gtk.IconSize.BUTTON)
        home_button.connect("clicked", lambda x: self.emit("jump-to-today"))
        home_button.set_sensitive(False)
        home_button.set_tooltip_text(_("Go to Today"))

        self.syncing_date = False
        self.calendar = calendar = Gtk.Calendar()
        calendar.connect("day-selected-double-click", self.on_day_selected)
        calendar.get_style_context().add_class("no-border")
        calendar.show()

        self.popover = popover = Gtk.Popover()
        popover.add(calendar)

        # Jump-to-selected-day button
        self.jump_button = jump_button = Gtk.MenuButton()
        image = Gtk.Image.new_from_icon_name("go-jump-symbolic", Gtk.IconSize.BUTTON)
        jump_button.connect("clicked", self.popup_calendar)
        jump_button.set_tooltip_text(_("Go to selected day"))
        jump_button.set_popover(popover)
        jump_button.set_image(image)

        home_box = Gtk.HBox()
        home_box.pack_start(home_button, False, False, 6)
        home_box.pack_start(jump_button, False, False, 6)

        self.pack_start(navigation_box, False, False, 6)
        self.pack_start(home_box, False, False, 6)

        self.view_index = 3 # FixMe: Switch to StackSwitcher later

    def sync_date(self, date):
        self.syncing_date = True
        self.calendar.select_month(date.month - 1, date.year)
        self.calendar.select_day(date.day)
        self.syncing_date = False

    def on_day_selected(self, calendar):
        if self.syncing_date:
            return

        self.emit("jump-to-selected-day", calendar.get_date())
        self.popover.popdown()

    def popup_calendar(self, button):
        self.popover.popup()

    def add_new_view_button(self, button, i=0):
        self.pack_start(button, False, False, 6)
        self.reorder_child(button, self.view_index)
        self.view_index += 1
        button.show()

class StockIconButton(Gtk.Button):
    def __init__(self, stock_id, label=None, size=Gtk.IconSize.BUTTON):
        super(StockIconButton, self).__init__()
        self.size = size
        self.set_alignment(0, 0)
        self.set_relief(Gtk.ReliefStyle.NONE)
        self.image = Gtk.Image.new_from_stock(stock_id, size)
        if not label:
            self.add(self.image)
        else:
            box = Gtk.HBox()
            self.label = Gtk.Label(label=label)
            box.pack_start(self.image, False, False, 2)
            box.pack_start(self.label, False, True, 0)
            self.add(box)

    def set_stock(self, stock_id):
        self.image.set_from_stock(stock_id, self.size)


#######################
# More Information Pane
##

class InformationBox(Gtk.VBox):
    """
    Holds widgets which display information about a uri

    obj: the content object to be shown
    is_molteplicity: bool used to discriminate grouped items from single ones
    """
    obj = None
    is_molteplicity = False

    class _ImageDisplay(Gtk.Image):
        """
        A display based on GtkImage to display a uri's thumb or icon using GioFile
        """
        def set_content_object(self, obj):
            if obj:
                if isinstance(obj, GioFile) and obj.has_preview():
                    pixbuf = obj.get_thumbnail(size=SIZE_NORMAL, border=3)
                    if pixbuf is None: pixbuf = obj.get_icon(size=64)
                else:
                    pixbuf = obj.get_icon(size=64)

                if pixbuf is None: pixbuf = obj.get_actor_pixbuf(size=64)
                self.set_from_pixbuf(pixbuf)

    def __init__(self):
        super(InformationBox, self).__init__()
        vbox = Gtk.VBox()
        self._box = Gtk.Frame()
        self.label = Gtk.Label()
        self.pathlabel = Gtk.Label()
        self.pathlabel.modify_font(Pango.FontDescription.from_string("Monospace 7"))
        self.box_label = Gtk.EventBox()
        self.box_label.add(self.pathlabel)
        self.box_label.connect("button-press-event", self.on_path_label_clicked)
        self.box_label.connect("enter-notify-event", self.on_path_label_enter)
        self.box_label.connect("leave-notify-event", self.on_path_label_leave)
        self.box_label.connect("realize", self.on_realize_event)
        labelvbox = Gtk.VBox()
        labelvbox.pack_start(self.label, False, True, 0)
        labelvbox.pack_end(self.box_label, False, True, 0)
        self.pack_start(labelvbox, True, True, 5)
        self._box.set_shadow_type(Gtk.ShadowType.NONE)
        vbox.pack_start(self._box, True, True, 0)
        self.pathlabel.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
        self.label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
        self.add(vbox)
        self.display_widget = self._ImageDisplay()
        self._box.add(self.display_widget)
        self.show_all()

    def set_displaytype(self, obj):
        """
        Determines the ContentDisplay to use for a given uri
        """
        self.display_widget.set_content_object(obj)
        self.show_all()

    def set_content_object(self, obj):
        self.obj = obj
        self.set_displaytype(obj)
        text = get_text_or_uri(obj,molteplicity=self.is_molteplicity)
        self.label.set_markup("<span size='10336'>" + text + "</span>")
        path = obj.uri.replace("&", "&amp;").replace("%20", " ")
        self.is_file = path.startswith("file://")
        if self.is_file:
            self.textpath = os.path.dirname(path)[7:]
        else:
            if self.is_molteplicity:
                path = text.split(" ")[-1]  #take only the uri
            self.textpath = path

        self.pathlabel.set_markup("<span color='#979797'>" + self.textpath + "</span>")

    def on_realize_event(self, parms):
        if self.is_file:
            hand = Gdk.Cursor.new(Gdk.CursorType.HAND2)
            self.box_label.props.window.set_cursor(hand)

    def on_path_label_clicked(self, wid, e):
        if self.is_file:
            os.system('xdg-open "%s"' % self.textpath)

    def on_path_label_enter(self, wid, e):
        if self.is_file:
            self.pathlabel.set_markup("<span color='#970000'><b>" + self.textpath+ "</b></span>")

    def on_path_label_leave(self, wid, e):
        if self.is_file:
            self.pathlabel.set_markup("<span color='#999797'>" + self.textpath + "</span>")

class _RelatedPane(Gtk.TreeView):
    """
    . . . . .
    .       .
    .       . <--- Related files
    .       .
    . . . . .

    Displays related events using a widget based on Gtk.TreeView
    """
    def __init__(self, column_name):
        super(_RelatedPane, self).__init__()
        self.popupmenu = ContextMenu
        self.connect("button-press-event", self.on_button_press)
        self.connect("row-activated", self.row_activated)
        pcolumn = Gtk.TreeViewColumn(_(column_name))
        pcolumn.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        pcolumn.set_spacing(6)
        pixbuf_render = Gtk.CellRendererPixbuf()
        pcolumn.pack_start(pixbuf_render, False)
        pcolumn.set_cell_data_func(pixbuf_render, self.celldatamethod, "pixbuf")
        text_render = Gtk.CellRendererText()
        text_render.set_property("ellipsize", Pango.EllipsizeMode.MIDDLE)
        pcolumn.pack_end(text_render, True)
        pcolumn.set_cell_data_func(text_render, self.celldatamethod, "text")
        self.append_column(pcolumn)
        self.set_headers_visible(True)
        self.set_fixed_height_mode(True)

    def celldatamethod(self, column, cell, model, iter_, user_data):
        if model:
            obj = model.get_value(iter_, 0)
            if user_data == "text":
                text = get_text_or_uri(obj)
                cell.set_property("text", text)
            elif user_data == "pixbuf":
                cell.set_property("pixbuf", obj.icon)

    def _set_model_in_thread(self, structs):
        """
        A threaded which generates pixbufs and emblems for a list of structs.
        It takes those properties and appends them to the view's model
        """
        lock = threading.Lock()
        self.active_list = []
        liststore = Gtk.ListStore(GObject.TYPE_PYOBJECT)
        self.set_model(liststore)
        for struct in structs:
            if not struct.content_object: continue
            self.active_list.append(False)
            liststore.append((struct.content_object,))

    def set_model_from_list(self, structs):
        self.last_active = -1
        if not structs:
            self.set_model(None)
            return
        thread = threading.Thread(target=self._set_model_in_thread, args=(structs,))
        thread.start()

    def on_button_press(self, widget, event):
        if event.button == 3:
            path = self.get_path_at_pos(int(event.x), int(event.y))
            if path:
                model = self.get_model()
                obj = model[path[0]][0]
                self.popupmenu.do_popup(event.time, [obj])
        return False

    def row_activated(self, widget, path, col, *args):
        if path:
            model = self.get_model()
            if model:
                obj = model[path[0]][0]
                obj.launch()

class InformationContainer(Gtk.Dialog):
    """
    . . . . .
    .  URI  .
    . Info  .
    .       .
    . Tags  .
    . . . . .
    . . . . .
    .       .
    .       . <--- Related files
    .       .
    . . . . .

    A pane which holds the information pane and related pane
    """
    __gsignals__ = {
        "content-object-set":  (GObject.SignalFlags.RUN_LAST, None,()),
        }

    class _InformationToolbar(Gtk.Toolbar):
        def __init__(self):
            Gtk.Toolbar.__init__(self)
            self.set_icon_size(Gtk.IconSize.SMALL_TOOLBAR)
            self.open_button = ob = InformationToolButton(icon="document-open-symbolic")
            ob.set_label(_("Launch this subject"))
            self.delete_button = del_ = InformationToolButton(icon="edit-delete-symbolic")
            del_.set_label(_("Delete this subject"))
            self.pin_button = pin = InformationToolButton(icon="view-pin-symbolic")
            pin.set_label(_("Add Pin"))
            self.note_button = note = InformationToolButton(icon="document-edit-symbolic")
            note.set_label(_("Edit Note"))
            sep = Gtk.SeparatorToolItem()
            for item in (del_, note, pin, sep, ob):
                if item:
                    self.insert(item, 0)

    class _EditNoteWindow(Gtk.Window):

        def __init__(self, parent, obj):
            Gtk.Window.__init__(self)
            self.obj = obj
            self.set_title(_("Edit Note"))
            self.set_transient_for(parent)
            self.set_destroy_with_parent(True)
            self.set_size_request(400, 400)
            self.connect("destroy", self.on_save_note)
            sw = Gtk.ScrolledWindow()
            sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
            textview = Gtk.TextView()
            self.textbuffer = textview.get_buffer()
            self.textbuffer.set_text("" if self.obj.annotation is None else self.obj.annotation)
            sw.add(textview)
            self.add(sw)
            self.show_all()

        def on_save_note(self, *arg):
            self.obj.annotation = self.textbuffer.get_text(self.textbuffer.get_start_iter (),
                                                           self.textbuffer.get_end_iter(),
                                                           False)

    def __init__(self, parent=None):
        super(Gtk.Window, self).__init__()
        if parent: self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_size_request(400, 400)
        self.connect("destroy", self.hide_on_delete)
        box1 = Gtk.VBox()
        box2 = Gtk.VBox()
        vbox = Gtk.VBox()
        self.toolbar = self._InformationToolbar()
        self.infopane = InformationBox()
        self.relatedpane = _RelatedPane("Used With")
        scrolledwindow = Gtk.ScrolledWindow()
        box2.set_border_width(5)
        box1.pack_start(self.toolbar, False, False, 0)
        box2.pack_start(self.infopane, False, False, 4)
        scrolledwindow.set_shadow_type(Gtk.ShadowType.IN)
        scrolledwindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolledwindow.add(self.relatedpane)
        vbox.set_property("expand", True)
        vbox.pack_end(scrolledwindow, True, True, 0)
        scrolledwindow.set_size_request(50, 100)
        box2.pack_end(vbox, True, True, 10)
        box1.pack_start(box2, True, True, 0)
        area = self.get_content_area()
        area.add(box1)
        def _launch(w):
            self.obj.launch()
        self.toolbar.open_button.connect("clicked", _launch)
        self.toolbar.delete_button.connect("clicked", self.do_delete_events_with_shared_uri)
        self.toolbar.pin_button.connect("clicked", self.do_toggle_bookmark)
        self.toolbar.note_button.connect("clicked", self.do_edit_note)
        self.connect("size-allocate", self.size_allocate)
        # Remove the close button
        separator = Gtk.SeparatorToolItem()
        separator.set_expand(True)
        separator.set_draw(False)
        self.toolbar.insert(separator, -1)

    def size_allocate(self, widget, allocation):
        if allocation.height < 400:
            self.infopane.display_widget.hide()
        else:
            self.infopane.display_widget.show()

    def do_toggle_bookmark(self, *args):
        uri = str(self.obj.uri)
        if bookmarker.is_bookmarked(uri):
            bookmarker.unbookmark(uri)
        else:
            bookmarker.bookmark(uri)

    def do_edit_note(self, *args):
        window = self._EditNoteWindow(self, self.obj)

    def do_delete_events_with_shared_uri(self, *args):
        CLIENT.find_event_ids_for_template(
            Event.new_for_values(subject_uri=self.obj.uri),
            lambda ids: CLIENT.delete_events(list(map(int, ids))))
        self.hide()

    def set_content_object(self, obj):
        self.obj = obj
        if not isinstance(self.obj, GioFile):
            self.toolbar.note_button.set_sensitive(False)
        def _callback(events):
            self.relatedpane.set_model_from_list(events)
        get_related_events_for_uri(obj.uri, _callback)
        self.infopane.set_content_object(obj)
        self.set_title(_("More Information"))
        self.show()
        self.emit("content-object-set")

    def hide_on_delete(self, widget, *args):
        super(InformationContainer, self).hide_on_delete()
        return True

class MolteplicityInformationContainer(Gtk.Dialog):
    """
    . . . . .
    .Origin .
    . Info  .
    . . . . .
    . . . . .
    .       .
    .       . <--- Similar items (same basename)
    .       .
    . . . . .

    A pane which holds the information pane and grouped items pane
    """
    def __init__(self, parent=None):
        super(Gtk.Window, self).__init__()
        if parent: self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_size_request(400, 400)
        self.connect("destroy", self.hide_on_delete)
        box2 = Gtk.VBox()
        vbox = Gtk.VBox()
        vbox.props.expand = True
        self.infopane = InformationBox()
        self.infopane.is_molteplicity = True
        self.relatedpane = _RelatedPane("Grouped items")
        scrolledwindow = Gtk.ScrolledWindow()
        box2.set_border_width(5)
        box2.pack_start(self.infopane, False, False, 4)
        scrolledwindow.set_shadow_type(Gtk.ShadowType.IN)
        scrolledwindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolledwindow.add(self.relatedpane)
        vbox.pack_end(scrolledwindow, True, True, 0)
        scrolledwindow.set_size_request(50, 100)
        box2.pack_end(vbox, True, True, 10)
        area = self.get_content_area()
        area.add(box2)
        self.connect("size-allocate", self.size_allocate)

    def size_allocate(self, widget, allocation):
        if allocation.height < 400:
            self.infopane.display_widget.hide()
        else:
            self.infopane.display_widget.show()

    def set_item_list(self, list_):
        self.infopane.set_content_object(list_[0].content_object)
        self.relatedpane.set_model_from_list(list_)
        self.set_title(_("More Information"))
        self.show()

    def hide_on_delete(self, widget, *args):
        super(MolteplicityInformationContainer, self).hide_on_delete()
        return True


class PreferencesDialog(Gtk.Dialog):

    class _PluginTreeView(Gtk.TreeView):
        def __init__(self):
            Gtk.TreeView.__init__(self)
            self.set_grid_lines(Gtk.TreeViewGridLines.VERTICAL)
            self.set_headers_visible(False)
            acolumn = Gtk.TreeViewColumn("")
            toggle_render = Gtk.CellRendererToggle()
            acolumn.pack_start(toggle_render, False)
            acolumn.add_attribute(toggle_render, "active", 1)
            self.append_column(acolumn)

            bcolumn = Gtk.TreeViewColumn("")
            text_render = Gtk.CellRendererText()
            text_render.set_property("ellipsize", Pango.EllipsizeMode.MIDDLE)
            bcolumn.pack_start(text_render, True)
            bcolumn.add_attribute(text_render, "markup", 0)
            self.append_column(bcolumn)
            self.set_property("rules-hint", True)
            self.connect("row-activated" , self.on_activate)

        def set_state(self, entry, state):
            PluginManager.plugin_settings[entry.key] = state

        def on_activate(self, widget, path, column):
            model = self.get_model()
            model[path][1] = not model[path][1]
            self.set_state(model[path][2], model[path][1])
            bname = os.path.basename(model[path][2].key)
            if model[path][1]:
                self.manager.activate(name=bname)
            else:
                self.manager.deactivate(name=bname)

        def set_items(self, manager):
            entries = manager.plugin_settings["active-plugins"]
            store = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_BOOLEAN, GObject.TYPE_PYOBJECT)
            for entry in entries:
                bname = os.path.basename(entry.key)
                if bname in manager.plugins:
                    # Load the plugin if the plugin is found
                    module = manager.plugins[bname]
                    name = "<b>" + module.__plugin_name__ + "</b>"
                    desc = "\n<small>" + module.__description__ + "</small>"
                    store.append( [name+desc, False if entry.value is None else entry.value.get_bool(), entry])
                else:
                    # Remove the key if no plugin is found
                    entries.remove(entry)
                    manager.plugin_settings["active-plugins"] = entries
            self.manager = manager
            self.set_model(store)

    def __init__(self, parent=None):
        super(PreferencesDialog, self).__init__()
        if parent: self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_title(_("Preferences"))
        self.set_size_request(400, 500)
        area = self.get_content_area()
        self.notebook = notebook = Gtk.Notebook()
        self.notebook.props.expand = True
        area.pack_start(notebook, False, True, 0)
        notebook.set_border_width(10)
        #Plugin page
        plugbox = Gtk.VBox()
        plugbox.set_border_width(10)
        self.plug_tree = self._PluginTreeView()
        label = Gtk.Label(label= _("Active Plugins:"))
        label.set_alignment(0, 0.5)
        plugbox.pack_start(label, False, False, 4)
        scroll_win = Gtk.ScrolledWindow()
        scroll_win.set_shadow_type(Gtk.ShadowType.IN)
        scroll_win.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        scroll_win.add(self.plug_tree)
        plugbox.add(scroll_win)
        notebook.append_page(plugbox, Gtk.Label(label= _("Plugins")))
        #Configuration page
        vbox = Gtk.VBox()
        vbox.set_border_width(5)
        hbox_tray = Gtk.HBox(False, 12)
        hbox_tray.props.margin = 12
        label = Gtk.Label(label=_("Show icon in system tray"))
        self.check_button = Gtk.CheckButton()
        self.check_button.set_active(settings["tray-icon"])
        self.check_button.connect("toggled", self.on_check_toggled)
        hbox_tray.pack_start(self.check_button, False, False, 0)
        hbox_tray.pack_start(label, False, False, 0)
        vbox.pack_start(hbox_tray, False, False, 0)
        notebook.append_page(vbox, Gtk.Label(label= _("Configuration")))

        self.connect("delete-event", lambda *args: (True, self.hide())[0])
        close_button = Gtk.Button(stock=Gtk.STOCK_CLOSE)
        self.add_action_widget(close_button, Gtk.ResponseType.DELETE_EVENT)
        close_button.connect("clicked", lambda *args: (True, self.hide())[0])

    def on_check_toggled(self, button, *args):
        settings["tray-icon"] = button.get_active()


#Code adapted from Emesene2 source. Get it at: https://github.com/emesene/emesene
#Thanks Emesene's team
class NiceBar(Gtk.EventBox):
    '''A class used to display messages in a non-intrusive bar'''

    def __init__(self, default_background=None, default_foreground=None):

        Gtk.EventBox.__init__(self)

        self.style = self.get_style_context()
        self.style.add_class("nice-bar")

        self.NORMALFOREGROUND = "white"

        (found, warning_color) = self.style.lookup_color("warning_color")

        if found:
            self.ALERTBACKGROUND = warning_color
        else:
            self.ALERTBACKGROUND = Gdk.RGBA.parse("#f57900")

        (found, normal_color) = self.style.lookup_color("success_color")

        if found:
            self.NORMALBACKGROUND = normal_color
        else:
            self.NORMALBACKGROUND = Gdk.RGBA.parse("#33d17a")

        self.props.height_request = 40
        self.message_label = Gtk.Label()
        self.message_label.set_line_wrap(True)
        self.message_label.set_ellipsize(Pango.EllipsizeMode.END)
        self.message_image = Gtk.Image()
        self.message_hbox = Gtk.HBox()
        self.message_hbox.set_border_width(2)
        self.message_hbox.set_halign(Gtk.Align.CENTER)

        if default_background is None:
            default_background = self.NORMALBACKGROUND
        if default_foreground is None:
            default_foreground = self.NORMALFOREGROUND

        self.default_back = default_background
        self.default_fore = default_foreground
        self.empty_queue()
        self.markup = '<span foreground="%s" font-size="large">%s</span>'
        self.override_background_color(Gtk.StateFlags.NORMAL, default_background)

        self.message_hbox.pack_end(self.message_label, False, True, 0)
        self.add(self.message_hbox)

    def new_message(self, message, stock=None, background=None, \
                                                  foreground=None):
        ''' Adds the actual message to the queue and show a new one '''

        if self.actual_message != '':
            self.messages_queue.append([self.actual_message, \
                    self.actual_image, self.actual_background,
                    self.actual_foreground])

        self.display_message(message, stock, background, foreground)

    def remove_message(self, portal):
        ''' Removes the actual message and display the next if any '''
        try:
            message, stock, back, fore = self.messages_queue.pop()
            self.display_message(message, stock, back, fore)
        except IndexError:
            self.hide()

        if portal.nicebar_timeout:
            GObject.source_remove(portal.nicebar_timeout)
            portal.nicebar_timeout = None

        return GLib.SOURCE_REMOVE

    def display_message(self, message, stock=None, background=None, \
                        foreground=None):
        '''
            Displays a message without modifying the queue
            A background, text color and a stock image are optional
        '''

        self.actual_message = message
        self.actual_image = stock
        self.actual_background = background or self.default_back
        self.actual_foreground = foreground or self.default_fore

        if self.message_image.get_parent() is not None:
            self.message_hbox.remove(self.message_image)

        if stock is not None:
            self.message_image = Gtk.Image.new_from_stock(stock, \
                                             Gtk.IconSize.LARGE_TOOLBAR)
            self.message_hbox.pack_start(self.message_image, False, False, 0)

        self.override_background_color(Gtk.StateFlags.NORMAL, self.actual_background)
        self.message_label.set_markup(self.markup % (self.actual_foreground,
                                                       self.actual_message))
        self.show_all()

    def empty_queue(self):
        ''' Delets all messages and hide the bar '''

        self.messages_queue = list()
        self.actual_message = ''
        self.actual_image = None
        self.actual_background = self.default_back
        self.actual_foreground = self.default_fore
        self.hide()

###
if Gst is not None:
    VideoPreviewTooltip = VideoPreviewTooltip()
    AudioPreviewTooltip = AudioPreviewTooltip()
else:
    VideoPreviewTooltip = None
    AudioPreviewTooltip = None
StaticPreviewTooltip = StaticPreviewTooltip()
ContextMenu = ContextMenu()
ContextMenuMolteplicity = ContextMenuMolteplicity()
SearchBox = SearchBox()



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-

# Zeitgeist - GNOME Activity Journal Extension
#
# Copyright © 2010 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
import dbus
import logging

from _zeitgeist.engine import constants
from _zeitgeist.engine.extension import Extension
#from _zeitgeist.engine.sql import get_default_cursor

GAJ_DBUS_OBJECT_PATH = "/org/gnome/zeitgeist/journal/activity"

log = logging.getLogger("zeitgeist.activity_journal")

class ActivityJournalExtension(Extension, dbus.service.Object):
    """
    This is a specialized extension which adds a new :const:`GetHistogramData`
    method to Zeitgeist for use by GNOME Activity Journal.
    """
    PUBLIC_METHODS = ["get_histogram_data"]

    def __init__ (self, engine):
        Extension.__init__(self, engine)
        dbus.service.Object.__init__(self, dbus.SessionBus(),
            GAJ_DBUS_OBJECT_PATH)
        
        self._engine = engine
        self._cursor = engine._cursor
    
    # PUBLIC
    def get_histogram_data(self):
        """ 
        FIXME: Please write me!
        """
        sql =  """SELECT
                strftime('%s', datetime(timestamp/1000, 'unixepoch'), 'start of day') AS daystamp,
                COUNT(*)
            FROM event
            GROUP BY daystamp
            ORDER BY daystamp DESC"""
        return self._cursor.execute(sql).fetchall()
    
    @dbus.service.method(constants.DBUS_INTERFACE,
                        in_signature="",
                        out_signature="a(tu)")
    def GetHistogramData(self):
        """
        FIXME: I do stuff.
        """
        return self.get_histogram_data()



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# GNOME Activity Journal
#
# Copyright © 2010 Randal Barlow <email.tehk@gmail.com>
# Copyright © 2010 Markus Korn
# Copyright © 2010 Siegfried Gevatter <siegfried@gevatter.com>
# Copyright © 2020 The GNOME Activity Journal developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import cairo
import calendar
import gettext

from math import pi as PI
from .common import *

import gi
from gi.repository import GObject, Pango, Gtk, Gdk

def get_gc_from_colormap(widget, shade):
    """
    Gets a Gdk.GC and modifies the color by shade
    """
    gc = widget.style.text_gc[Gtk.StateFlags.INSENSITIVE]
    if gc:
        color = widget.style.text[4]
        color = shade_gdk_color(color, shade)
        gc.set_rgb_fg_color(color)
    return gc


class CairoHistogram(Gtk.DrawingArea):
    """
    A histogram which is represented by a list of dates, and nitems.

    There are a few maintenance issues due to the movement abilities. The widget
    currently is able to capture motion events when the mouse is outside
    the widget and the button is pressed if it was initially pressed inside
    the widget. This event mask magic leaves a few flaws open.
    """
    _selected = (0,)
    padding = 2
    bottom_padding = 23
    top_padding = 2
    wcolumn = 12
    xincrement = wcolumn + padding
    start_x_padding = 2
    max_width = xincrement
    column_radius = 0
    stroke_width = 1
    stroke_offset = 0
    min_column_height = 4
    max_column_height = 101
    gc = None
    pangofont = None
    _disable_mouse_motion = False
    selected_range = 0
    _highlighted = []
    _last_location = -1
    _single_day_only = False
    colors = {
        "bg" : (1, 1, 1, 1),
        "base" : (1, 1, 1, 1),
        "column_normal" :  (1, 1, 1, 1),
        "column_selected" : (1, 1, 1, 1),
        "column_alternative" : (1, 1, 1, 1),
        "column_selected_alternative" : (1, 1, 1, 1),
        "font_color" : "#ffffff",
        "stroke" : (1, 1, 1, 0),
        "shadow" : (1, 1, 1, 0),
        }

    _store = None

    __gsignals__ = {
        "selection-set" : (GObject.SignalFlags.RUN_LAST, None,
                           (GObject.TYPE_PYOBJECT,)),
        "data-updated" : (GObject.SignalFlags.RUN_LAST, None,()),
        "column_clicked" : (GObject.SignalFlags.RUN_LAST, None,
                            (GObject.TYPE_PYOBJECT,))
        }
    _connections = {"style-updated": "change_style",
                    "draw": "_draw",
                    "button_press_event": "mouse_press_interaction",
                    "motion_notify_event": "mouse_motion_interaction",
                    "key_press_event": "keyboard_interaction",
                    "scroll-event" : "mouse_scroll_interaction",
    }
    _events = (Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.BUTTON_MOTION_MASK |
                  Gdk.EventMask.POINTER_MOTION_HINT_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK |
                  Gdk.EventMask.BUTTON_PRESS_MASK)

    def __init__(self):
        """
        :param datastore: The.CairoHistograms two dimensional list of dates and nitems
        :param selected_range: the number of days displayed at once
        """
        super(CairoHistogram, self).__init__()

        self.style = self.get_style_context()
        self.style.add_class("cairo-histogram")

        self._selected = []
        self.set_events(self._events)
        self.set_can_focus(True)
        for key, val in self._connections.items():
            self.connect(key, getattr(self, val))
        self.font_name = self.get_pango_context().get_font_description().get_family()

    def change_style(self, widget):
        """
        Sets the widgets style and coloring
        """
        self.colors = self.colors.copy()
        self.colors["bg"] = get_gtk_rgba(self.style, "bg", 0)
        self.colors["base"] = get_gtk_rgba(self.style, "base", 0)

        shade = 0.8 if is_light_theme() else 2.8
        self.colors["column_normal"] = get_gtk_rgba(self.style, "text", 4, shade)
        self.colors["column_selected"] = get_gtk_rgba(self.style, "bg", 3)
        color = get_color_by_name(self.style, "theme_bg_color")
        fcolor = get_color_by_name(self.style, "theme_fg_color")
        c = combine_gdk_color(color, fcolor)
        self.colors["font_color"] = get_hex_color(c)

        # use custom color for highlighting search results in histogram
        color = get_histogram_search_color(self.style)
        self.colors["column_alternative"] = color

        self.colors["column_selected_alternative"] = get_gtk_rgba(self.style, "bg", 3, 0.6)
        self.colors["stroke"] = get_gtk_rgba(self.style, "text", 4, 0.80)
        self.colors["shadow"] = get_gtk_rgba(self.style, "text", 4, 0.80)
        self.font_size = self.get_pango_context().get_font_description().get_size()/1024
        self.pangofont = Pango.FontDescription.from_string(self.font_name + " %d" % self.font_size)
        self.pangofont.set_weight(Pango.Weight.BOLD)
        self.bottom_padding = self.font_size + 9 + 6 #widget.style.ythickness

    def set_store(self, store):
        self._store = store
        self.largest = min(max(max([len(x) for x in store.days]), 1), 200)
        if not self.get_selected():
            self.set_selected([datetime.date.today()])
        else:
            self.set_selected(self.get_selected())
        self.queue_draw()
        self.last_updated = time.time()

    def get_store(self):
        return self._store

    def _draw(self, widget, context):
        """
        The major drawing method that the draw event calls directly
        """
#        widget.style.set_background(widget.window, Gtk.StateFlags.NORMAL)
        self.draw(widget, context)

    def draw(self, widget, context):
        """
        The minor drawing method

        :param context: This drawingarea's cairo context from the draw event
        """

        if not self.pangofont:
            self.pangofont = Pango.FontDescription.from_string(self.font_name + " %d" % self.font_size)
            self.pangofont.set_weight(Pango.Weight.BOLD)

        context.set_source_rgba(*self.colors["base"])
        context.set_operator(cairo.Operator.SOURCE)
        self.width = self.get_allocated_width()
        self.height = self.get_allocated_height()

        context.set_operator(cairo.Operator.SOURCE)
        #context.paint()
        context.rectangle(0, 0, self.width, self.height)
        context.clip()
        #context.set_source_rgba(*self.colors["bg"])
        context.rectangle(0, 0, self.width, self.height - self.bottom_padding)
        context.fill()
        self.draw_columns_from_store(context, self.get_selected())
        context.set_line_width(1)
        if type(self) == CairoHistogram:
            Gtk.render_frame(self.style, context,
                             0, 0,
                             self.width, self.height - self.bottom_padding)
        if self.is_focus():
            Gtk.render_focus(self.style, context,
                             0, 0,
                             self.width, self.height - self.bottom_padding)

    def draw_columns_from_store(self, context, selected):
        """
        Draws columns from a datastore

        :param context: This drawingarea's cairo context from the draw event
        :param selected: a list of the selected dates
        """
        x = self.start_x_padding
        months_positions = []
        for day in self.get_store().days:
            if day.date.day == 1:
                months_positions += [(day.date, x)]
            if day.date in self._highlighted:
                color = self.colors["column_selected_alternative"] if day.date in selected else self.colors["column_alternative"]
            elif day.date in selected:
                color = self.colors["column_selected"]
            else:
                color = self.colors["column_normal"]
            self.draw_column(context, x, self.height, len(day), color)
            x += self.xincrement
        if x > self.width: # Check for resize
            self.set_size_request(x+self.xincrement, self.height)

        for date, xpos in months_positions:
            edge = 0
            if (date, xpos) == months_positions[-1]:
                edge = len(self._store)*self.xincrement
            self.draw_month(context, xpos - self.padding, self.height, date, edge)
        self.max_width = x # remove me

    def draw_column(self, context, x, maxheight, nitems, color):
        """
        Draws a columns at x with height based on nitems, and maxheight

        :param context: The drawingarea's cairo context from the draw event
        :param x: The current position in the image
        :param maxheight: The event areas height
        :param nitems: The number of items in the column to be drawn
        :param color: A RGBA tuple Example: (0.3, 0.4, 0.8, 1)
        """
        if nitems > self.max_column_height:
            nitems = self.max_column_height
        maxheight = maxheight - self.bottom_padding - 2
        height = int((((float(nitems)/self.largest)**0.33)*(maxheight-2))) - self.top_padding
        if height < self.min_column_height:
            height = self.min_column_height
        y = maxheight - height
        context.set_source_rgba(*color)
        context.move_to(x + self.column_radius, y)
        context.new_sub_path()
        if nitems > 4:
            context.arc(self.column_radius + x, self.column_radius + y, self.column_radius, PI, 3 * PI /2)
            context.arc(x + self.wcolumn - self.column_radius, self.column_radius + y, self.column_radius, 3 * PI / 2, 0)
            context.rectangle(x, y + self.column_radius, self.wcolumn, height - self.column_radius)
        else:
            context.rectangle(x, y, self.wcolumn, height)
        context.close_path()
        context.fill()

    def draw_month(self, context, x, height, date, edge=0):
        """
        Draws a line signifying the start of a month
        """
        context.set_source_rgba(*self.colors["stroke"])
        context.set_line_width(self.stroke_width)
        context.move_to(x+self.stroke_offset, 0)
        context.line_to(x+self.stroke_offset, height - self.bottom_padding)
        context.stroke()
        month = calendar.month_name[date.month]
        date = "<span color='%s'>%s %d</span>" % (self.colors["font_color"], month, date.year)
        layout = self.create_pango_layout(date)
        layout.set_markup(date)
        layout.set_font_description(self.pangofont)
        w, h = layout.get_pixel_size()
        if edge:
            if x + w > edge: x = edge - w - 5
        Gtk.render_layout(self.style, context, int(x + 3), int(height - self.bottom_padding/2 - h/2), layout)

    def set_selected(self, dates):
        if dates == self._selected:
            return False
        self._selected = dates
        if dates:
            date = dates[-1]
        self.emit("selection-set", dates)
        self.queue_draw()
        return True

    def get_selected(self):
        """
        returns a list of selected indices
        """
        return self._selected

    def clear_selection(self):
        """
        clears the selected items
        """
        self._selected = []
        self.queue_draw()

    def set_highlighted(self, highlighted):
        """
        Sets the widgets which should be highlighted with an alternative color

        :param highlighted: a list of indexes to be highlighted
        """
        if isinstance(highlighted, list):
            self._highlighted = highlighted
        else: raise TypeError("highlighted is not a list")
        self.queue_draw()

    def clear_highlighted(self):
        """Clears the highlighted color"""
        self._highlighted = []
        self.queue_draw()

    def set_single_day(self, choice):
        """
        Allows the cal to enter a mode where the trailing days are not selected but still kept
        """
        self._single_day_only = choice
        self.queue_draw()

    def get_store_index_from_cartesian(self, x, y):
        """
        Gets the datastore index from a x, y value
        """
        return int((x - self.start_x_padding) / self.xincrement)

    def keyboard_interaction(self, widget, event):
        if event.keyval in (Gdk.KEY_space, Gdk.KEY_Right, Gdk.KEY_Left, Gdk.KEY_BackSpace):
            i = self.get_selected()
            if isinstance(i, list) and len(i) > 0: i = i[-1]
            if event.keyval in (Gdk.KEY_space, Gdk.KEY_Right):
                i = i + datetime.timedelta(days=1)
            elif event.keyval in (Gdk.KEY_Left, Gdk.KEY_BackSpace):
                i = i + datetime.timedelta(days=-1)
            if i < datetime.date.today() + datetime.timedelta(days=1):
                self.change_location(i)

    def mouse_motion_interaction(self, widget, event, *args, **kwargs):
        """
        Reacts to mouse moving (while pressed), and clicks
        """
        #if (event.get_state() == Gdk.ModifierType.BUTTON1_MASK and not self._disable_mouse_motion):
        location = min((self.get_store_index_from_cartesian(event.x, event.y), len(self._store.days) - 1))
        if location != self._last_location:
            self.change_location(location)
            self._last_location = location
            #return True
        return False

    def mouse_press_interaction(self, widget, event, *args, **kwargs):
        if (event.y > self.get_size_request()[1] - self.bottom_padding and
            event.y < self.get_size_request()[1]):
            return False
        location = min((self.get_store_index_from_cartesian(event.x, event.y), len(self._store.days) - 1))
        if location != self._last_location:
            self.change_location(location)
            self._last_location = location
        return True

    def mouse_scroll_interaction(self, widget, event):
        date = self.get_selected()[-1]
        i = self.get_store().dates.index(date)
        if (event.direction in (Gdk.ScrollDirection.UP, Gdk.ScrollDirection.RIGHT)):
            if i+1< len(self.get_store().days):
                self.change_location(i+1)
        elif (event.direction in (Gdk.ScrollDirection.DOWN, Gdk.ScrollDirection.LEFT)):
            if 0 <= i-1:
                self.change_location(i-1)

    def change_location(self, location):
        """
        Handles click events
        """
        if isinstance(location, int):
            if location < 0:
                return False
            store = self.get_store()
            date = store.days[location].date
        else: date = location
        self.emit("column_clicked", date)
        return True

    left_icon = get_icon_for_name("back", 16)
    right_icon = get_icon_for_name("forward", 16)
    def _expose_scroll_buttons(self, widget, event):
        render_pixbuf(widget.window, event.area.x + event.area.width-16, event.area.y+event.area.height-16, self.right_icon, False)
        render_pixbuf(widget.window, event.area.x, event.area.y+event.area.height-16, self.left_icon, False)


def _in_area(coord_x, coord_y, area):
    """check if some given X,Y coordinates are within an area.
    area is either None or a (top_left_x, top_left_y, width, height)-tuple"""
    if area is None:
        return False
    area_x, area_y, area_width, area_height = area
    return (area_x <= coord_x <= area_x + area_width) and \
        (area_y <= coord_y <= area_y + area_height)


def _in_area(coord_x, coord_y, area):
    """check if some given X,Y coordinates are within an area.
    area is either None or a (top_left_x, top_left_y, width, height)-tuple"""
    if area is None:
        return False
    area_x, area_y, area_width, area_height = area
    return (area_x <= coord_x <= area_x + area_width) and \
        (area_y <= coord_y <= area_y + area_height)


class TooltipEventBox(Gtk.EventBox):
    """
    A event box housing the tool tip logic that can be used for a CairoHistogram.
    Otherwise it interferes with the scrubbing mask code
    """
    _saved_tooltip_location = None
    def __init__(self, histogram, container):
        super(TooltipEventBox, self).__init__()
        self.add(histogram)
        self.histogram = histogram
#        self.container = container FixMe: revisit this
        self.set_property("has-tooltip", True)
        self.connect("query-tooltip", self.query_tooltip)

    def query_tooltip(self, widget, x, y, keyboard_mode, tooltip):
        if y < self.histogram.get_size_request()[1] - self.histogram.bottom_padding:
            location = self.histogram.get_store_index_from_cartesian(x, y)
            if location != self._saved_tooltip_location:
                # don't show the previous tooltip if we moved to another
                # location
                self._saved_tooltip_location = location
                return False
            try:
                day = self.histogram.get_store().days[location]
                count = len(day)
            except IndexError:
                # there is no bar for at this location
                # don't show a tooltip
                return False
            date = day.date.strftime("%A, %d %B, %Y")
            tooltip.set_text("%s\n%i %s" % (date, count,
                                            gettext.ngettext("item", "items", count)))
        else:
            return False
        return True


class JournalHistogram(CairoHistogram):
    """
    A subclass of CairoHistogram with theming to fit into Journal
    """
    padding = 2
    column_radius = 1.3
    top_padding = 6
    bottom_padding = 29
    wcolumn = 10
    xincrement = wcolumn + padding
    column_radius = 2
    stroke_width = 2
    stroke_offset = 1
    font_size = 12
    min_column_height = 2

    def change_style(self, widget):
        self.colors = self.colors.copy()
        self.colors["bg"] = get_gtk_rgba(self.style, "bg", 0)
        self.colors["color"] = get_gtk_rgba(self.style, "base", 0)
        self.colors["column_normal"] =  get_gtk_rgba(self.style, "bg", 1)
        self.colors["column_selected"] = get_gtk_rgba(self.style, "bg", 3)
        self.colors["column_selected_alternative"] = get_gtk_rgba(self.style, "bg", 3, 0.7)
        self.colors["column_alternative"] = get_gtk_rgba(self.style, "text", 2)
        self.colors["stroke"] = get_gtk_rgba(self.style, "bg", 0)
        self.colors["shadow"] = get_gtk_rgba(self.style, "bg", 0, 0.98)
        self.font_size = self.style.font_desc.get_size()/1024
        self.bottom_padding = self.font_size + 9 + widget.style.ythickness
        self.pangofont = Pango.FontDescription.from_string(self.font_name + " %d" % self.font_size)
        self.pangofont.set_weight(Pango.Weight.BOLD)


class HistogramWidget(Gtk.ScrolledWindow):
    """
    A container for a CairoHistogram which allows you to scroll
    """
    __gsignals__ = {
        # the index of the first selected item in the datastore.
        "date-changed" : (GObject.SignalFlags.RUN_LAST, None,
                           (GObject.TYPE_PYOBJECT,)),
        }

    def __init__(self, histo_type=CairoHistogram):
        """
        :param histo_type: a :class:`CairoHistogram <CairoHistogram>` or a derivative
        """
        super(HistogramWidget, self).__init__()
        screen = self.get_screen()
        if screen.get_width() < 1280:
            size = (600, 55)
        else:
            size = (600, 75)

        self.set_shadow_type(Gtk.ShadowType.NONE)
        self.histogram = histo_type()
        self.eventbox = TooltipEventBox(self.histogram, self)
        self.set_size_request(*size)
        self.add(self.eventbox)
        self.histogram.connect("column_clicked", self.date_changed)
        self.histogram.connect("selection-set", self.scrubbing_fix)
        self.histogram.queue_draw()
        self.queue_draw()
        self.connect("size-allocate", self.on_resize)
        self.histogram.connect("button-press-event", self.mouse_click_scroll)

    def mouse_click_scroll(self, widget, event):
        hadjustment = self.get_hadjustment()
        value = hadjustment.get_value()
        if event.x - value < 16:
            hadjustment.set_value(max(0, value-10))
        elif event.x > value + hadjustment.get_page_size() - 16:
            hadjustment.set_value(min(self.histogram.max_width, value+10))
        self.histogram.queue_draw()

    def on_resize(self, widget, allocation):
        dates = self.histogram.get_selected()
        self.scrubbing_fix(self.histogram, dates)

    def date_changed(self, widget, date):
        self.emit("date-changed", date)

    def set_store(self, store):
        self.histogram.set_store(store)
        self.scroll_to_end()

    def set_dates(self, dates):
        self.histogram.set_selected(dates)

    def scroll_to_end(self, *args, **kwargs):
        """
        Scroll to the end of the drawing area's viewport
        """
        hadjustment = self.get_hadjustment()
        hadjustment.set_value(1)
        hadjustment.set_value(self.histogram.max_width - hadjustment.get_page_size())

    def scrubbing_fix(self, widget, dates):
        """
        Allows scrubbing to scroll the scroll window
        """
        if not len(dates):
            return
        store = widget.get_store()
        i = store.dates.index(dates[0])
        hadjustment = self.get_hadjustment()
        proposed_xa = ((i) * self.histogram.xincrement) + self.histogram.start_x_padding
        proposed_xb = ((i + len(dates)) * self.histogram.xincrement) + self.histogram.start_x_padding
        if proposed_xa < hadjustment.get_value():
            hadjustment.set_value(proposed_xa)
        elif proposed_xb > hadjustment.get_value() + hadjustment.get_page_size():
            hadjustment.set_value(proposed_xb - hadjustment.get_page_size())



#!/usr/bin/env python3
#
# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# GNOME Activity Journal - Installation script
#
# Copyright © 2009-2010 Siegfried Gevatter <siegfried@gevatter.com>
# Copyright © 2020 The GNOME Activity Journal developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys
from glob import glob

try:
    from distutils.core import setup
    from DistUtilsExtra.command import *
except ImportError as e:
    print("ERROR: %s." % e)
    print()
    print("Please install 'python3-distutils' and 'python3-distutils-extra' packages.")
    print()
    print("See the README.md file for more information.")
    sys.exit(1)

class _build_i18n(build_i18n.build_i18n):

    def run(self):
        # Generate POTFILES.in from POTFILES.in.in
        if os.path.isfile("po/POTFILES.in.in"):
            lines = []
            with open("po/POTFILES.in.in") as f:
                for line in f:
                    line = line.strip()
                    expanded = glob(line)
                    if not expanded:
                        expanded = [line]
                    lines.extend(expanded)
            with open("po/POTFILES.in", "w") as f:
                f.write("\n".join(lines) + "\n")
        build_i18n.build_i18n.run(self)

def recursive_install(dst, directory, prefix=None, ext=None):
    l = []
    this = []
    for name in glob('%s/*' % directory):
        if os.path.isdir(name):
            l.extend(recursive_install(dst, name, os.path.join(prefix,
                os.path.basename(name)) if prefix is not None else None))
        elif not ext or os.path.splitext(name)[1] in ext:
            this.append(name)
    l.append((os.path.join(dst,
        prefix if prefix is not None else directory), this))
    return l

def list_from_lists(*args):
    l = []
    for arg in args:
        l.extend(arg)
    return l

try:
    VERSION = [line for line in open('src/config.py')
        if line.startswith('VERSION')][0].split('"')[1]
except (IOError, IndexError):
    VERSION = 'unknown'

setup(
    name = 'gnome-activity-journal',
    version = VERSION,
    description = 'GUI to browse and search your Zeitgeist activities',
    long_description = \
        'GNOME Activity Journal is a tool for easily browsing and finding '\
        'files on your computer. It shows a chronological journal of all '\
        'file activity and supports full-text search through Tracker.',
    author = 'GNOME Activity Journal Developers',
    author_email = 'zeitgeist@lists.launchpad.net',
    url = 'https://wiki.gnome.org/Apps/ActivityJournal',
    license = 'GPLv3+',
    platforms = ['GNU/Linux'],
    data_files = list_from_lists(
        [('bin/', ['gnome-activity-journal'])],
        [('share/gnome-activity-journal/data', glob('data/*.svg'))],
        [('share/gnome-activity-journal/data', glob('data/*.png'))],
        [('share/gnome-activity-journal/data', glob('data/*.css'))],
        [('share/gnome-activity-journal/data/ui', glob('data/ui/*.ui'))],
        [('share/gnome-activity-journal/data/zlogo', glob('data/zlogo/*.png'))],
        [('share/pixmaps/', ['data/org.gnome.ActivityJournal.xpm'])],
        recursive_install('share/icons/hicolor', 'data/icons/hicolor/', '',
            ext=['.png', '.svg']),
        recursive_install('share/gnome-activity-journal', 'src/', ext=['.py']),
        [('share/man/man1/', ['extra/gnome-activity-journal.1'])],
        [('share/zeitgeist/_zeitgeist/engine/extensions/', glob('extension/*.py'))],
        ),
    cmdclass = {
        'build': build_extra.build_extra,
        'build_i18n': _build_i18n,
        #'build_help':  build_help.build_help,
        #'build_icons': build_icons.build_icons,
        },
    classifiers = [
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI-Approved :: GNU General Public License (GPL)',
        'Intended Audience :: End Users/Desktop',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Environment :: X11 Applications :: GTK',
        ]
    )



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# GNOME Activity Journal
#
# Copyright © 2009-2010 Seif Lotfy <seif@lotfy.com>
# Copyright © 2009-2010 Siegfried Gevatter <siegfried@gevatter.com>
# Copyright © 2007 Alex Graveley <alex@beatniksoftware.com>
# Copyright © 2010 Markus Korn <thekorn@gmx.de>
# Copyright © 2010 Randal Barlow <email.tehk@gmail.com>
# Copyright © 2020 The GNOME Activity Journal developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pickle
import gettext
import os
import sys
import urllib.request, urllib.parse, urllib.error
from xdg import BaseDirectory

from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, \
    ResultType

import gi
from gi.repository import Gio, GObject

# Installation details
BASE_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
DATA_PATH = os.path.join(BASE_PATH, "data")
NAME = "GNOME Activity Journal"
VERSION = "1.0.0"
ZEITGEIST_REQUIRED_VERSION=[1, 0, 3]
GETTEXT_PATH = os.path.join(BASE_PATH, "../locale")

USER_DATA_PATH = BaseDirectory.save_data_path("gnome-activity-journal")

PLUGIN_PATH = os.path.join(BASE_PATH, "src/plugins")
if not os.path.exists(PLUGIN_PATH) or not os.path.isdir(PLUGIN_PATH):
    PLUGIN_PATH = None
USER_PLUGIN_PATH = os.path.join(USER_DATA_PATH, "plugins")
if not os.path.exists(USER_PLUGIN_PATH) or not os.path.isdir(USER_PLUGIN_PATH):
    USER_PLUGIN_PATH = None

settings = Gio.Settings.new("org.gnome.ActivityJournal")
preference_settings = Gio.Settings.new("org.gnome.ActivityJournal.preferences")

DEBUG = False
SCALE_TO_CENTER = preference_settings["scale-to-center-thumbnail"]
SHOW_THUMBNAILS_IN_TIMELINE = preference_settings["show-thumbnails-in-timeline"]
SHOW_DAY_NIGHT_GRADIENT_IN_TIMELINE = preference_settings["show-day-night-gradient-in-timeline"]
CONFIRM_ITEM_DELETION = preference_settings["confirm-item-deletion"]
HIDE_EMPTY_DAY_LABELS = preference_settings["hide-empty-day-labels"]
SEPARATE_ZOOM_PER_VIEW = preference_settings["separate-zoom-per-view"]

SEARCH_IN_SUBJECT_URI = preference_settings["search-in-subject-uri"]
SEARCH_IGNORE_SINGLE_CHAR = preference_settings["search-ignore-single-char"]
SEARCH_IS_CASE_SENSITIVE = preference_settings["search-is-case-sensitive"]
SEARCH_RESULTS_USE_BOLD_STYLE = preference_settings["search-results-use-bold-style"]
SEARCH_RESULTS_USE_LARGE_STYLE = preference_settings["search-results-use-large-style"]
SEARCH_RESULTS_USE_DEFAULT_COLOR = preference_settings["search-results-use-default-color"]
SEARCH_RESULTS_TEXT_COLOR = preference_settings["search-results-text-color"]
SEARCH_RESULTS_HISTOGRAM_COLOR = preference_settings["search-results-histogram-color"]
SEARCH_RESULTS_TEXT_COLOR_UPDATED = preference_settings["search-results-text-color-updated"]
SEARCH_RESULTS_HISTOGRAM_COLOR_UPDATED = preference_settings["search-results-histogram-color-updated"]

SHOW_AUDIO_PREVIEW = preference_settings["show-audio-preview"]
SHOW_VIDEO_PREVIEW = preference_settings["show-video-preview"]

# day parts
CHANGEHOURS = [ 0 ]
CHANGEHOURS.append(preference_settings["day-part-early-morning-to"])
CHANGEHOURS.append(preference_settings["day-part-morning-to"])
CHANGEHOURS.append(preference_settings["day-part-afternoon-to"])
CHANGEHOURS.append(preference_settings["day-part-evening-to"])

#FIXME i know it's ugly. Btw this is only a temp solution. It will be removed
# when we'll revamp the Multiview view.----sorry----cando
IN_ERASE_MODE = False

def _get_path(path):
    return os.path.join(BASE_PATH, path)

def get_data_path(path=None):
    return os.path.join(DATA_PATH, path) if path else DATA_PATH

def get_icon_path(path):
    for basepath in (DATA_PATH, "/usr/share/", "/usr/local/share",
        os.path.expanduser("~/.local/share")):
        newpath = os.path.join(basepath, "icons", path)
        if os.path.exists(newpath):
            return newpath
    return None

def event_exists(uri):
    # TODO: Move this into Zeitgeist's datamodel.py
    return not uri.startswith("file://") or os.path.exists(
        urllib.parse.unquote(str(uri[7:])))

# When running from Bazaar, give priority to local translations
if os.path.isdir(_get_path("build/mo")):
    GETTEXT_PATH = _get_path("build/mo")


class Bookmarker(GObject.GObject):

    __gsignals__ = {
        "reload" : (GObject.SignalFlags.RUN_FIRST,
                    None,
                    (GObject.TYPE_PYOBJECT,))
        }
    # PUBLIC!
    bookmarks = []
    def __init__(self):
        GObject.GObject.__init__(self)
        self.bookmarks_file = os.path.join(USER_DATA_PATH, "bookmarks.pickled")
        self._load()

    def _load(self):
        if os.path.isfile(self.bookmarks_file):
            try:
                with open(self.bookmarks_file, "rb") as f:
                    self.bookmarks = pickle.load(f)
                    removable = []
                    for bookmark in self.bookmarks:
                        if not event_exists(bookmark):
                            removable.append(bookmark)
                    for uri in removable:
                        self.bookmarks.remove(uri)
            except Exception:
                print("Pin database is corrupt.")

    def _save(self):
        with open(self.bookmarks_file, "wb") as f:
            pickle.dump(self.bookmarks, f)

    def bookmark(self, uri):
        if not uri in self.bookmarks and event_exists(uri):
            self.bookmarks.append(uri)
        self._save()
        self.emit("reload", self.bookmarks)

    def unbookmark(self, uri):
        if uri in self.bookmarks:
            self.bookmarks.remove(uri)
        self._save()
        self.emit("reload", self.bookmarks)

    def is_bookmarked(self, uri):
        return uri in self.bookmarks

class PluginManager(object):
    """
    Loads a module and calls the module's activate(client, store, window) function

    Where:
    client is a zeitgeist client
    store is a the backing Store which controls journal
    window is the Journal window

    All plugins must have:
    func activate(client, store, window): build up when the plugin is enabled
    func deactivate(client, store, window): tear down when the plugin is disabled
    str __plugin_name__: plugin name
    str __description__: description of the plugin
    """
    plugin_settings = Gio.Settings.new("org.gnome.ActivityJournal.plugins")

    def __init__(self, client, store, window):
        self.plugins = {}
        self.client = client
        self.store = store
        self.window = window
        # Base plugins
        self.load_path(PLUGIN_PATH)
        self.load_path(USER_PLUGIN_PATH)

    def load_path(self, path):
        if path:
            sys.path.append(path)
            plugs = []
            for module_file in os.listdir(path):
                if module_file.endswith(".py") and module_file != "__init__.py":
                    modname = module_file.replace(".py", "").replace("-", "_")
                    plugs.append(modname)
            self.get_plugins(plugs, level=0)

    def get_plugins(self, plugin_names, prefix="", level=-1):
        plugs = self.import_plugins(plugin_names, prefix=prefix, level=level)
        self.load_plugins(plugs)

    def import_plugins(self, plugs, prefix="", level=-1):
        plugins = []
        for plugin_name in plugs:
            try:
                plugin_module = __import__(prefix + plugin_name, level=level, fromlist=[plugin_name])
                plugins.append((plugin_name, plugin_module))
                self.plugins[plugin_name] = plugin_module
                # print  plugin_module.__plugin_name__ + " has been imported"
            except Exception as e:
                print(" Importing %s failed." % plugin_name, e)
        return plugins

    def load_plugins(self, plugins):
        for plugin_name, plugin_module in plugins:
            try:
                # FixMe: plugin code is very primitive. Replace with
                # libpeas / libpeasgtk, and remove all plugin code
                # later.

                # state = self.plugin_settings[plugin_name]
                # if not state: continue # If the plugin is not True it will not be loaded
                self.activate(plugin_module)
            except Exception as e:
                print("Loading %s failed." % plugin_name, e)

    def __get_plugin_from_name(self, plugin=None, name=None):
        if not plugin:
            plugin = self.plugins[name]
        elif not plugin and not name:
            raise TypeError("You must pass either a plugin or a plugin_name")
        return plugin

    def activate(self, plugin=None, name=None):
        plugin = self.__get_plugin_from_name(plugin, name)
        plugin.activate(self.client, self.store, self.window)
        print("Activating %s" % plugin.__plugin_name__)

    def deactivate(self, plugin=None, name=None):
        plugin = self.__get_plugin_from_name(plugin, name)
        plugin.deactivate(self.client, self.store, self.window)
        print("Deactivating %s" % plugin.__plugin_name__)



# Singletons and constants
bookmarker = Bookmarker()



# -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
#
# from src import common # For common functions used by journal
# from src import supporting_widgets to use the non view related widgets journal uses throughout


__plugin_name__ = "Example Plugin"
# Enter a detailed description here
__description__ = "An example plugin, which does nothing important"


def activate(client, store, window):
    """
    This function is called to activate the plugin.

    :param client: the zeitgeist client used by journal
    :param store: the date based store which is used by journal to handle event and content object request
    :param window: the activity journal primary window
    """
    print("Activate")


def deactivate(client, store, window):
    """
    This function is called to activate the plugin.

    :param client: the zeitgeist client used by journal
    :param store: the date based store which is used by journal to handle event and content object request
    :param window: the activity journal primary window
    """
    print("Deactivate")
